#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>  /*function close(...)*/

#include <Xm/ArrowB.h>    /*motif libraries...*/
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/Text.h>
#include <Xm/ToggleBP.h>
#include <Xm/MwmUtil.h>
#include <Xm/TextF.h>

#include "control.h"
#include "graphics.h"
#include "callback.h"
#include "fitsigma.h"
#include "simu.h"
#include "calc.h"
#include "gvar.h"

/* global variables needed in callback.c */

XtWorkProcId id; /*needed in stopping and restarting the simu*/
XtIntervalId arrowbtimer;
extern Container c[];

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/
/*Sets the text-arrow widget taw to sensitivity truth*/
void TASensitive(int taw,int truth)
{
  XtSetSensitive(c[taw].text, truth);
  XtSetSensitive(c[taw].arrow1, truth);
  XtSetSensitive(c[taw].arrow2, truth);
}

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/
/*
 * text widget value changed callback
 */
void CbTextValueChanged(Widget w, XtPointer clientData, XtPointer callData)
{
  int idn, temp;   
  char buff[6];
  static int lastavrcutvalue;  /*needed in avrcut text widget*/     
  idn = (int)(long)clientData;  /*get the correct widget*/ 
  *c[idn].value = atof(XmTextGetString(w));
   
  /* check that the new value is in the allowed range*/
  if (*c[idn].value < c[idn].min)      *c[idn].value = c[idn].min;
  else if (*c[idn].value > c[idn].max) *c[idn].value = c[idn].max;
  
  /*a long if to handle different text widgets*/
  if (idn == SIZE_X){
    simArea.x = (1.0 + edge.x / 100.0) * (*c[idn].value);
    /* Simulation cell size has to be adjusted if periodic bc in x */
    adjustsimarea(periodicXFlag,&(simArea.x),lParam);
    /**c[ZOOM_X].value = *c[idn].value + 1;
    sprintf(buff, "%g", *c[ZOOM_X].value);
    XmTextSetString(c[ZOOM_X].text, buff);*/
  } 
  else if (idn == SIZE_Y){
    simArea.y = (1.0 + edge.y / 100.0) * (*c[idn].value);
    /**c[ZOOM_Y].value = *c[idn].value + 1;
    sprintf(buff, "%g", *c[ZOOM_Y].value);
    XmTextSetString(c[ZOOM_Y].text, buff);*/
  }
  
  if (idn == ZOOM_X || idn == SIZE_X) {
    temp = (int)((simArea.x - zoom.x) / 0.2) + 5;
    if (temp < 0) temp = 0;
    if (center.x > temp / 10) {
      XtVaSetValues(wHScrollB, XmNvalue, temp, NULL);
      center.x = (double)temp / 10.0;
    } 
    else if (center.x < -temp / 10) {
      XtVaSetValues(wHScrollB, XmNvalue, -temp, NULL);
      center.x = -(double)temp / 10.0;
    }
    XtVaSetValues(wHScrollB,
		  XmNminimum,     -temp,
		  XmNmaximum,     temp + 100,
		  NULL);
  } 
  else if (idn == ZOOM_Y || idn == SIZE_Y) {
    temp = (int)((simArea.y - zoom.y) / 0.2) + 5;
    if (temp < 0)
      temp = 0;
    if (center.y > temp / 10) {
      XtVaSetValues(wVScrollB, XmNvalue, temp, NULL);
      center.y = (double)temp / 10.0;
    } 
    else if (center.y < -temp / 10) {
      XtVaSetValues(wVScrollB, XmNvalue, -temp, NULL);
      center.y = -(double)temp / 10.0;
    }
    XtVaSetValues(wVScrollB,
		  XmNminimum,     -temp,
		  XmNmaximum,     temp + 100,
		  NULL);
  }
  /*The following 2 else ifs cause that avrCut can be 0 or >=1 but 
    not for example 0.1 because that would consume huge amounts of memory 
    without a reason.*/
  else if (idn == AVRCUT && *c[idn].value >= 1.0) {
    lastavrcutvalue=avrCut;
    VSCopy(avCells, 1.0 / *c[idn].value, simArea);
    ALLOC(avr, avCells.x * avCells.y, VecD);
    ALLOC(avn, avCells.x * avCells.y, int);
    ALLOC(avatype,avCells.x * avCells.y, int);
    ALLOC(avv, avCells.x * avCells.y, VecD);
  }
  else if (idn == AVRCUT && *c[idn].value < 1.0) {
    if (avrCut<=lastavrcutvalue && avrCut>0) {
      avrCut=0;
      XmTextSetString(c[idn].text, "0");
    }
    else if (avrCut>lastavrcutvalue) {
      avrCut=1;
      XmTextSetString(c[idn].text, "1");
    }
    
    lastavrcutvalue=avrCut;
    *c[idn].value=avrCut;
    VSCopy(avCells, avrCut, simArea);  
    ALLOC(avr, avCells.x * avCells.y, VecD);
    ALLOC(avn, avCells.x * avCells.y, int);
    ALLOC(avatype,avCells.x * avCells.y, int);
    ALLOC(avv, avCells.x * avCells.y, VecD);
  }
  
  /* 
    Check if we have to build the system anew.
    If you add new text widgets you probably have 
    to change the if clause. See control.h for details.
  */
  if (idn < STRAIN_P) CalcAtoms();
  Redraw();
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/


/*
 * text widget modify verification callback
 * this is called when a new character is typed in a text widget
 */
void CbTextModifyVerify(Widget w, XtPointer clientData, XtPointer callData)
{
  int allow,cbnum;
  char *v;
  XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *)callData;
  cbnum = (long int) clientData;
  switch (c[cbnum].nmdvalue) {
  case TAN:
    allow = ALLOW_NUMBERS;
    break;
  case TAND:
    allow = ALLOW_NUMBERS | ALLOW_DOT;
    break;
  case TANMD:
    allow = ALLOW_NUMBERS | ALLOW_MINUS | ALLOW_DOT;
    break;
  case TANM:
    allow = ALLOW_NUMBERS | ALLOW_MINUS;
    break;
  }
  
  v = ((XmTextVerifyCallbackStruct *)callData)->text->ptr;
  
  /* If the typed character is allowed, return immediately.
     Allowed characters are indicated by different bits of clientData. */
  if (cbs->text->length == 0) {
    return;
  }
  /*else {
    if (cbs->startPos>=c[cbnum].length) {
      cbs->doit = False;
      return;
    }
  }*/
  if (v == NULL) return;
  else if (v[0] == 0) /* back space is always allowed */
    return;
  else if ((allow & ALLOW_NUMBERS) && v[0] >= '0' && v[0] <= '9')
    return;
  else if ((allow & ALLOW_MINUS) && v[0] == '-')
    return;
  else if ((allow & ALLOW_DOT) && v[0] == '.')
    return;
  else
    ((XmTextVerifyCallbackStruct *)callData)->doit = False;
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * text widget lose focus callback
 */
void CbTextLoseFocus(Widget w, XtPointer clientData, XtPointer callData)
{
  
  double temp;
  char buff[6];
  int id;

  id = (int)(long)clientData;
  temp = atof(XmTextGetString(w));
  if (temp != *c[id].value) {
    sprintf(buff, "%g", *c[id].value);
    XmTextSetString(w, buff);
  }
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*arrowbutton widget's callback. This handles holding down the button which
  causes the values to change without clicking. This calls CbArrow which
  makes the main job. All this procedure does is to stop changing the value
  when the button disarms and to call cbarrow when the button arms*/

void CbArrowTimerStart(Widget wid,XtPointer client_data,XtPointer call_data)
{
  XmArrowButtonCallbackStruct *cbs = (XmArrowButtonCallbackStruct *)
    call_data;
  if (cbs->reason == XmCR_ARM)
    CbArrow(client_data,1);
  else if (cbs->reason == XmCR_DISARM)
    XtRemoveTimeOut(arrowbtimer);
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * arrowbutton widget's callback. CbTextValueChanged is the main callback
 * this doesn't have that much to do, just update the value if that is
 * allowed. This is called only when a textwidget is updated by pressing
 * the arrows. CbTextValueChanged is called always when the text changes,
 * whatever the cause. This procedure also starts a timer which handles
 * changing the textarrowwidget's value constantly when pressing the mouse
 * button.
 */
void CbArrow(XtPointer clientData, XtIntervalId timerid)
{
  char buff[6];
  double temp;
  int id;
  
  id = (int)(long)clientData / 2;
  /*a switch case to check all necessary textwidgets.*/ 
  switch (id) {
  case ROT_R:
  case BND_D:
  case RELAXATION:
  case DISP_X:
  case DISP_Y: 
    if (!boundaryFlag) return;
    break;    /*these widgets not in use if no boundary*/

  case CRACK_X:
  case CRACK_Y:
  case CRACK_SX:
  case CRACK_SY:
  case CRACK_ROT: 
    if (!crackFlag) return;
    break;    /*these widgets not in use without crack*/

  case COOLTEMP:
  case SLOPE:
  case STARTTIME: 
    break;     

  case DOT:
    CalcAtoms();
  case UPCUTA:
  case DOWNCUTA: 
  case UPCUT:
  case DOWNCUT:
    break;

  case WAIT:     
    break;

  case AVRCUT:  
    if (sinfo != SVECT) return;
    break;

  case BEGINAT:
  case ENDAT:   
    if (!ani.limitsFlag) return;
    break;
  }

  /*check that the new value is in the allowed range*/
  temp = *c[id].value + (((long)clientData%2==0L)?-1.0:1.0)*c[id].step;
  if (temp < c[id].min) {
    temp = c[id].min;
    return;
  }
  else if (temp > c[id].max) {
    temp = c[id].max;
    return;
  }
  if (temp != *c[id].value) {
    sprintf(buff, "%g", temp);
    XmTextSetString(c[id].text, buff);   /*update the text to max/min value*/
    /*start the timer. On the first time (timerid==1) wait longer before
      repeating the callback*/
    arrowbtimer=XtAppAddTimeOut(app,timerid==1? 500 : 50,
				(XtTimerCallbackProc)CbArrow,clientData); 
  }
}



/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * Button callback - both pushb and toggleb. This is called always when a
 * sensitive button is pressed.
 */
void CbButton(Widget w, XtPointer clientData, XtPointer callData)
{
  char buff1[MAXLINE],buff2[MAXLINE];
  /* a switch case for all buttons that do something*/
  switch ((long)clientData) {
 
  case BOUNDARY: 
    boundaryFlag = !boundaryFlag;
    /*XtSetSensitive(c[ROT_R].text, boundaryFlag);
    XtSetSensitive(c[BND_D].text, boundaryFlag);*/
    CalcAtoms();
    Redraw();
    break;
  
  case VERTIC:   
    verticalFlag= !verticalFlag;
    if (verticalFlag && periodicXFlag) {
      XtVaSetValues(wPerxToggle, XmNset,False, NULL);
      XtCallCallbacks(wPerxToggle,XmNvalueChangedCallback,NULL);
    }  
    
    XtSetSensitive(wPerxToggle, !verticalFlag);
        
    CalcAtoms();
    Redraw();
    break;
    
  case SUGGEST:   
    suggestFlag= !suggestFlag;
    break;

  case PERX:
    periodicXFlag =! periodicXFlag;
    /*if (periodicXFlag) 
      edge.x=0.0;
    else
      edge.x=wDrawSize.x;
    simArea.x  = (1.0 + edge.x / 100.0) * size.x;
    simArea.y  = (1.0 + edge.y / 100.0) * size.y;
    adjustsimarea(periodicXFlag,&(simArea.x),lParam);*/
    CalcSimArea();
    CalcAtoms();
    Redraw();
    break;

  case CRACK: 
    crackFlag = !crackFlag;
    TASensitive(CRACK_X, crackFlag); 
    TASensitive(CRACK_Y, crackFlag); 
    TASensitive(CRACK_SX, crackFlag); 
    TASensitive(CRACK_SY,  crackFlag); 
    TASensitive(CRACK_ROT,  crackFlag); 
    CalcAtoms();
    Redraw();
    break;

  case TAPPLY:
    tapplyFlag = !tapplyFlag;
    break;
  case TADJUST:
    tadjustFlag = !tadjustFlag;
    break; 
  case TADJUST2:
    tadjust2Flag = !tadjust2Flag;
    break; 
  case TSAVE:
    savetscheme = !savetscheme;
    break; 
  
  case RUN: 
    if (runningFlag)
      XtRemoveWorkProc(id);
    else
      id = XtAppAddWorkProc(app, MdCycle, 0);
    runningFlag = !runningFlag; /* toggle run state */
    XtVaSetValues(wRunPushB, XmNalignment,XmALIGNMENT_CENTER,XtVaTypedArg,
		  XmNlabelString, T_STRING(runningFlag ? "Pause" : "Run"),
		  NULL);
    break;
    
  case PARAM_OK: 
    XtPopdown(XtParent(wParamMan));
    if(wWinSeamFlag) XtPopdown(XtParent(wSeamMan));
    strainFlag = False;
    CalcAtoms();
    InitRun();
    XtSetSensitive(wRunPushB, True);
    if (tapplyFlag) {
      readTscheme(tfname); 
      checkTscheme();
    }
    if (savetscheme) {
      if (! (tsavefile = fopen ("save.events", "w"))){
	fprintf(stderr,"Could not open file save.events\n");
	bexit(-1);
      }
      fprintf(tsavefile,"#\n#Auto-saved partial event list\n#\n");
      fprintf(tsavefile,"Size %.2f %.2f\n\n",size.x,size.y);
    }
    Redraw();
    RedrawPlot(1);
    break;

  case CANCEL:   
    XtPopdown(XtParent(wParamMan));
    if(wWinSeamFlag) XtPopdown(XtParent(wSeamMan));
    XtSetSensitive(wRunPushB, True);
    SaveParams(LOAD);
    Redraw();
    break; 

  case SEAM_OK: 
    XtPopdown(XtParent(wSeamMan));
    XtSetSensitive(wSeamPushB, True);
    wWinSeamFlag=0;
    break;
    
  case SEAM:  
    wWinSeamFlag=1;
    /*XtSetSensitive(wSeamPushB, False);*/
    OpenDialog(wSeamMan,w,"seam");
    break;
         
  
/*-----------------------------------------------*/

  case STRAIN: 
    strainStep = 0.005 * strainP * (xMax - xMin) / 
      (strainT / (1000 * deltaT));
    XtSetSensitive(wStrainPushB, False);
    XtSetSensitive(wFixedToggle, False);
    TASensitive(STRAIN_T, False);
    TASensitive(STRAIN_P, False);
    strainFlag = True;
    break;
    
  case FIXED_TOGGLE: 
    fixedFlag = !fixedFlag;
    XtSetSensitive(wStrainPushB, fixedFlag && runningFlag);
    Redraw();
    break;

  case CLOSE_STRAIN:   
    XtPopdown(XtParent(wStrainMan));
    /*Redraw();*/
    break;

/*-----------------------------------------------*/

  case TSCALE_TOGGLE: 
    cool.TScaleFlag = !cool.TScaleFlag;
    XtVaSetValues(wLowTempToggle,XmNset,cool.TScaleFlag,NULL); 
    break;
    
  case QUENCH_TOGGLE: 
    cool.QuenchFlag = !cool.QuenchFlag;
    XtVaSetValues(wQuenchToggle,XmNset,cool.QuenchFlag,NULL); 
    break;

  case RANDOMIZE:
    RandomizeVelocities();
    break;
  
  case START_COOL: 
    InitRun();
    cool.programon = 1;
    strainFlag = False;
    Redraw();
    RedrawPlot(1);
    if (runningFlag)
      XtRemoveWorkProc(id);
    else
      id = XtAppAddWorkProc(app, MdCycle, 0);
    runningFlag = !runningFlag; /* toggle run state */
    /*XtSetSensitive(wStrainPushB,
      runningFlag && !strainFlag && fixedFlag);*/
    XtSetSensitive(wCoolPushB,False);
    TASensitive(COOLTEMP, False);
    TASensitive(STARTTIME, False);
    TASensitive(SLOPE, False);
    
    XtVaSetValues(wRunPushB,  XmNalignment,XmALIGNMENT_CENTER,
		  XtVaTypedArg,
		  XmNlabelString, T_STRING(runningFlag ? "Pause" : "Run"),
		  NULL);
    Redraw();
    break;
    
  case CLOSE_TEMP:   
    XtPopdown(XtParent(wTempMan));
    break;
    
    /*-----------------------------------------------*/
  case MODIFY:
    ModifyArea(r,&nAtom,modarea);
    if (savetscheme) {
      switch (modinfo) {
      case M_MODRM:
	sprintf(buff1, "remove");
	break;
      case M_MODSTOP:
	sprintf(buff1, "stop");
	break;
      case M_MODTYPE1:
      sprintf(buff1,"type1");
	break;
      case M_MODTYPE2:
	sprintf(buff1, "type2");
	break;	
      case M_MODVEL:
	sprintf(buff1, "vel");
	break;
      case M_MODFIX:
	sprintf(buff1,"fix");
	break;
      case M_MODFORCE:
      sprintf(buff1, "force");
	break;
      case M_MODRMF:
	sprintf(buff1,"rmfix");
	break;
      }	
      fprintf(tsavefile,
	      "ModifyArea %.2f %s  %.2f %.2f  %.2f %.2f  %.2f %.2f  %.2f\n",
	      (stepCount * 1000 *deltaT), buff1, modvel.x,modvel.y, 
	      modarea.center.x, modarea.center.y,
	      modarea.size.x,modarea.size.y,modarea.angle); 
    }    
    Redraw();
    /*}*/
    break;

  case CLOSE_MOD:   
   XtPopdown(XtParent(wModMan));
   wWinModFlag = False;
   break;

  case MODSHOWSEL:
    showselFlag = !showselFlag;
    Redraw();
    break;
   /*-----------------------------------------------*/
  case CLOSE_ZOOM:   
    XtPopdown(XtParent(wZoomMan));
    break;

 /*-----------------------------------------------*/

  case CLOSE_COLOR:   
    XtPopdown(XtParent(wColorMan));
    break;

  case COLRANUPD:   
    colorranges.Update = !colorranges.Update;
    if (!colorranges.Update) {
      sprintf(buff1, "%.2g", colorranges.EkMin);
      sprintf(buff2, "%.2g", colorranges.EkMax);
      XmTextSetString(c[COLRANKMIN].text, buff1);
      XmTextSetString(c[COLRANKMAX].text, buff2);
      
      sprintf(buff1, "%.2g", colorranges.EpMin);
      sprintf(buff2, "%.2g", colorranges.EpMax);
      XmTextSetString(c[COLRANPMIN].text, buff1);
      XmTextSetString(c[COLRANPMAX].text, buff2);
      
      sprintf(buff1, "%.2g", colorranges.SMin);
      sprintf(buff2, "%.2g", colorranges.SMax);
      XmTextSetString(c[COLRANSMIN].text, buff1);
      XmTextSetString(c[COLRANSMAX].text, buff2);

      sprintf(buff1, "%.2g", colorranges.FMin);
      sprintf(buff2, "%.2g", colorranges.FMax);
      XmTextSetString(c[COLRANFMIN].text, buff1);
      XmTextSetString(c[COLRANFMAX].text, buff2);
    }
    TASensitive(COLRANKMIN, !colorranges.Update);
    TASensitive(COLRANKMAX, !colorranges.Update);
    TASensitive(COLRANPMIN, !colorranges.Update);
    TASensitive(COLRANPMAX, !colorranges.Update);
    TASensitive(COLRANSMIN, !colorranges.Update);
    TASensitive(COLRANSMAX, !colorranges.Update);
    TASensitive(COLRANFMIN, !colorranges.Update);
    TASensitive(COLRANFMAX, !colorranges.Update);
    break;

  case COLRANHIDE:   
    colorranges.hide = !colorranges.hide;
    Redraw();
    break;
    /*-----------------------------------------------*/

  case SETTRACKTOG: 
    SetTrackFlag = !SetTrackFlag; 
    TASensitive(TRANGE, SetTrackFlag);
    TASensitive(TFACTOR, SetTrackFlag);
    TASensitive(TBFACTOR, SetTrackFlag);
    XtSetSensitive(wSaveTrackToggle, SetTrackFlag);   
    XtSetSensitive(wShowTrackToggle, SetTrackFlag);   
    break;

  case SAVETRK:
    saveTrackFlag=!saveTrackFlag;
    if (saveTrackFlag){
      FileFlag=True;
    }
    else 
      if (FileFlag){
	StartFlag=False;
      }
    break;  
    
  case SHOWTRK:
    showTrackFlag=!showTrackFlag;
    break;  
    
  case CLOSE_TRK: 
    XtPopdown(XtParent(wTrackMan));
    wWinTrackFlag=False;
    break;

/*-----------------------------------------------*/
  
  case SET_ANI: 
    ani.aniFlag = !ani.aniFlag;
    break;
        
  case CLOSE_ANI: 
    XtPopdown(XtParent(wAniMan));
    wWinAniFlag = False;
    break;
 
  case LIM_TOGGLE: 
    ani.limitsFlag = !ani.limitsFlag; 
    TASensitive(BEGINAT, ani.limitsFlag);
    TASensitive(ENDAT, ani.limitsFlag);
    XtSetSensitive(wEndendToggle, ani.limitsFlag);
    break;
    
  case ENDEND: 
    ani.endendFlag = !ani.endendFlag;
    break; 
  
  }
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/
/*returns 1/true if atom i is in the area, else 0/false*/
int Selected(long int i, RArea area)
{
  double dx,dy,angle;

  angle=area.angle*Pi/180.0;
  dx=(r[i].x-area.center.x)*cos(angle)+(r[i].y-area.center.y)*sin(angle);
  dy=(r[i].y-area.center.y)*cos(angle)-(r[i].x-area.center.x)*sin(angle);
  if ((fabs(dx) <= 0.5*area.size.x) 
      && (fabs(dy) <= 0.5*area.size.y))
    return 1;
  else
    return 0;
  /*dx=r[i].x-area.center.x;
  dy=r[i].y-area.center.y;
  if ((dx <= 0.5*area.size.x) 
    && (dy <= 0.5*area.size.y))
    return 1;
  else
    return 0;*/ 
}

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/* Modifies atoms in a given area */
void ModifyArea(VecD *r, int *nAtom, RArea area)
{
  int i;
  double dx,dy;
  
  /*area.angle *= Pi / 180.0;*/

  for (i = 0; i < *nAtom; ++i) {
    /*dx=(r[i].x-x)*cos(area.angle)+(r[i].y-y)*sin(area.angle);
    dy=(r[i].y-y)*cos(area.angle)-(r[i].x-x)*sin(area.angle);*/
    if /*(fabs(dx) < 0.5*sizex && fabs(dy) < 0.5*sizey)*/(Selected(i,area)) {
      switch (modinfo) {
      case M_MODSTOP:
	rv[i].x = 0.0;
	rv[i].y = 0.0;
	break;
      case M_MODTYPE1:
	atype[i] = 1;
	break;
      case M_MODTYPE2:
	atype[i] = 2;
	break;
      case M_MODVEL:
	rv[i].x = rv[i].x+modvel.x;
	rv[i].y = rv[i].y+modvel.y;
	break;
      case M_MODFIX:
	fixvel[i]=1;
	rv[i].x = modvel.x;
	rv[i].y = modvel.y;
	cforce[i].x = 0.0;
	cforce[i].y = 0.0;
	break;
      case M_MODRMF:
	fixvel[i]=0;
	cforce[i].x = 0.0;
	cforce[i].y = 0.0;
	break;
      case M_MODFORCE:
	fixvel[i]=0;
	cforce[i].x = modvel.x;
	cforce[i].y = modvel.y;
	break;
      case M_MODRM:
	(*nAtom)--;
	r[i].x = r[*nAtom].x;
	r[i].y = r[*nAtom].y;
	rv[i].x = rv[*nAtom].x;
	rv[i].y = rv[*nAtom].y;
	atype[i]=atype[*nAtom];
	fixvel[i]=fixvel[*nAtom];
	i--;
	break;
      }
    }
  }
}

/*---------------------------------------------------------*/
/*---------------------------------------------------------*/

/* Saves atom positions to disk. This is almost directly called when the user
   selects "Save snapshot" from the file menu. "Almost" means that CbMenu is
   called but it doesn't do anything else but calls this one.*/
void SnapShot()
{
  Window dumpId;
  char cmd_string[200];
  static int filenum = 0;  
  double tfield;
  int i;
  FILE *cd;
  char bitfile_name[11],paramfile_name[25],line[MAXLINE],temp[4];
  /*AK*/
  static char aname[5][3];
  int ity;
  
  /* 
               atom      atom 
     atype[]   type      name    
     -------  --------   ----
       1      1 free      SI
       2      2 free      GE
      -1      1 fixed     GA
      -2      2 fixed     AS

  */
  strcpy(aname[0],"AS\0");
  strcpy(aname[1],"GA\0");
  strcpy(aname[2],"??\0");
  strcpy(aname[3],"SI\0");
  strcpy(aname[4],"GE\0");
  filenum += 1;
  
  /*checks if there are earlier snapshots and doesn't overwrite them*/
  system("ls coords.*.x?? 2>/dev/null | sort -n -r > tmxfllst.txt");
  cd=fopen("tmxfllst.txt","r");
  if (fgets(line,MAXLINE,cd)!=NULL)  {
    temp[0]=line[7];temp[1]=line[8];temp[2]=line[9];temp[3]=line[10];
    filenum=atol(temp)+1;
  }
  fclose(cd);
  system("rm tmxfllst.txt 2>/dev/null");
  
  /*save boundary params with the snapshot so that a situation can be reloaded
   later without changing the parameters*/
  sprintf(paramfile_name,"boundary.%04d.param",filenum);
  ROWParams("write",paramfile_name);
  
  if (PDBFORMAT) sprintf(bitfile_name,"coords.%04d.pdb",filenum);
  else sprintf(bitfile_name,"coords.%04d.xyz",filenum);
  if (! (cd = fopen (bitfile_name, "w"))){
    fprintf(stderr,"Could not open file %s: ",bitfile_name);
    perror("");
    fprintf(stderr,"Bexiting.\n");
    bexit(-1);
  }
   
  fprintf(stderr,"%d %s %ld %g\n",filenum,bitfile_name,
	  stepCount,deltaT*stepCount);
  
  if (PDBFORMAT) { /* Dump PDB in format with potEn as 'temperature' field */
    fprintf(cd,"COMPND    Frame number %ld %g\n",stepCount,deltaT*stepCount);
    for(i=0;i<nAtom;i++) {
      if (sinfo == SPOT) 
	tfield=TFIELDSCALING*potEn[i];
      else
	tfield=TFIELDSCALING*stressij[i];    
      ity=atype[i]+2; 
      fprintf(cd,"ATOM  %5d%3s   MOL     1    %8.3f%8.3f%8.3f%6.2f%6.2f\n",
	      i+1,aname[ity],r[i].x,r[i].y,0.0,1.0,tfield);
    }
    fprintf(cd,"TER   %5d\n",nAtom+1);
    fprintf(cd,"END\n");
  } 
  else { /* Dump XYZ in format */
    fprintf(cd,"%d\n",nAtom);
    fprintf(cd,"Frame number %ld %g\n",stepCount,deltaT*stepCount);
    
    for(i=0;i<nAtom;i++){
      if (sinfo == SPOT) 
	tfield=TFIELDSCALING*potEn[i];
      else
	tfield=TFIELDSCALING*stressij[i];    
      ity=atype[i]+2; 
      fprintf(cd,"%s %f %f %f %g\n",aname[ity],r[i].x,r[i].y,0.0,tfield); 
    }
  }

  fclose(cd);
  
  if (STOREVELOCITIES) {
    sprintf(bitfile_name,"velocities.%04d.dat",filenum);
    if (! (cd = fopen (bitfile_name, "w"))){
      fprintf(stderr,"Could not open file %s: ",bitfile_name);
      perror("");
      fprintf(stderr,"Bexiting.\n");
      bexit(-1);
    } 
    for(i=0;i<nAtom;i++) fprintf(cd,"%d %g %g %g %g\n",i,r[i].x,
				 r[i].y,rv[i].x,rv[i].y);
    fclose(cd);
  }
  
  if (DUMPWINDOW) { /* Dump drawing area to a xwd file */
    dumpId=XtWindow(wDraw);
    sprintf(cmd_string, "xwd -frame -out dump.%04d.xwd -id 0x%x",
	    filenum,(unsigned int)dumpId);
    system(cmd_string);
  }
}



/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * temporarily save and load initial parameters. Used by OK and cancel 
 * buttons in the parameters dialog.
 */
void SaveParams(int function)
{
  /*all parameters that are saved and loaded.*/
  static VecD sizeTemp, simAreaTemp;
  static RArea crackTemp;
  static Seamdata seamTemp;
  static double lParamTemp,lParam2Temp;
  static double cutTemp, borderWTemp,epsilonTemp;
  static int boundaryFlagTemp, suggestFlagTemp,periodicXFlagTemp;
  static VecD *rTemp;
  static int *atypeTemp;
  static int *borderatomTemp;
  static int nAtomTemp;
  static int savetschemeTemp,tapplyFlagTemp,tadjustFlagTemp;
  
  if (function == STORE) {
    sizeTemp.x = size.x;
    sizeTemp.y = size.y;
    simAreaTemp.x = simArea.x;
    simAreaTemp.y = simArea.y;
    seamTemp = seam;
    crackTemp = crack;
    lParamTemp = lParam;
    lParam2Temp = lParam2;
    epsilonTemp = epsilon;
    cutTemp = cut;
    borderWTemp = borderW;
    boundaryFlagTemp = boundaryFlag;
    suggestFlagTemp = suggestFlag;
    periodicXFlagTemp = periodicXFlag;
    savetschemeTemp=savetscheme;
    tapplyFlagTemp=tapplyFlag;
    tadjustFlagTemp=tadjustFlag;
    rTemp = r;
    atypeTemp = atype;
    borderatomTemp = borderatom;
    nAtomTemp = nAtom;
    r = NULL;
    atype = NULL;
    borderatom = NULL;
  }
  else {
    free(r);
    free(atype);
    free(borderatom);
    r = rTemp;
    atype = atypeTemp;
    borderatom = borderatomTemp;
    nAtom = nAtomTemp;
    size.x = sizeTemp.x;
    size.y = sizeTemp.y;
    simArea.x = simAreaTemp.x;
    simArea.y = simAreaTemp.y;
    seam=seamTemp;
    crack = crackTemp;
    lParam = lParamTemp;
    lParam2 = lParam2Temp;
    epsilon = epsilonTemp;
    cut = cutTemp;
    borderW = borderWTemp;
    boundaryFlag = boundaryFlagTemp;
    periodicXFlag = periodicXFlagTemp;
    suggestFlag=suggestFlagTemp;
    savetscheme=savetschemeTemp;
    tapplyFlag=tapplyFlagTemp;
    tadjustFlag=tadjustFlagTemp;
  }
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * scale widget's value-changed callback. Only 3 scales in the program
 * currently so this is a short one.
 */
void CbScale(Widget w, XtPointer clientData, XtPointer callData)
{
  int v,temp;
  v = ((XmScaleCallbackStruct *)callData)->value;
  switch ((long)clientData) {
  case DIAM: diam = (double)v / 100.0;
    break;
  case UPDATE: updateStep = v;
    break;
  case PLOTSCALE: plotdata.xscale = (double)v / 1.0;
    break;
  case ZOOMSCALE: 
    zoom.y = (double)v;
    zoom.x=zoom.y*wDrawSize.x/wDrawSize.y;
    
    temp = (int)((simArea.x - zoom.x) / 0.2) + 5;
    if (temp < 0) temp = 0;
    if (center.x > temp / 10) {
      XtVaSetValues(wHScrollB, XmNvalue, temp, NULL);
      center.x = (double)temp / 10.0;
    } 
    else if (center.x < -temp / 10) {
      XtVaSetValues(wHScrollB, XmNvalue, -temp, NULL);
      center.x = -(double)temp / 10.0;
    }
    XtVaSetValues(wHScrollB,
		  XmNminimum,     -temp,
		  XmNmaximum,     temp + 100,
		  NULL);
        
    temp = (int)((simArea.y - zoom.y) / 0.2) + 5;
    if (temp < 0)
      temp = 0;
    if (center.y > temp / 10) {
      XtVaSetValues(wVScrollB, XmNvalue, temp, NULL);
      center.y = (double)temp / 10.0;
    } 
    else if (center.y < -temp / 10) {
      XtVaSetValues(wVScrollB, XmNvalue, -temp, NULL);
      center.y = -(double)temp / 10.0;
    }
    XtVaSetValues(wVScrollB,
		  XmNminimum,     -temp,
		  XmNmaximum,     temp + 100,
		  NULL);
    
    break;
  }
  Redraw();
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * scroll bar's value-changed callback (draw area scrollbars)
 */
void CbScroll(Widget w, XtPointer clientData, XtPointer callData)
{
  if((int)(long)clientData == HORIZONTAL)
    center.x = (double)((XmScrollBarCallbackStruct *)callData)->value / 10.0;
  else
    center.y = (double)((XmScrollBarCallbackStruct *)callData)->value / 10.0;

  Redraw();
}

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

void UpdateScrollBars()
{
  int  temp = (int)((simArea.x - zoom.x) / 0.2) + 5;
  if (temp < 0) temp = 0;
  if (center.x > temp / 10) {
    XtVaSetValues(wHScrollB, XmNvalue, temp, NULL);
    center.x = (double)temp / 10.0;
  } 
  else if (center.x < -temp / 10) {
    XtVaSetValues(wHScrollB, XmNvalue, -temp, NULL);
    center.x = -(double)temp / 10.0;
  }
  XtVaSetValues(wHScrollB,
		  XmNminimum,     -temp,
		  XmNvalue,(int)(center.x*10.0),
             	  XmNmaximum,     temp + 100,
		  NULL);
        
  temp = (int)((simArea.y - zoom.y) / 0.2) + 5;
  if (temp < 0)
    temp = 0;
  if (center.y > temp / 10) {
    XtVaSetValues(wVScrollB, XmNvalue, temp, NULL);
    center.y = (double)temp / 10.0;
  } 
  else if (center.y < -temp / 10) {
    XtVaSetValues(wVScrollB, XmNvalue, -temp, NULL);
    center.y = -(double)temp / 10.0;
  }
  XtVaSetValues(wVScrollB,
		  XmNminimum,     -temp,
                  XmNvalue,(int)(center.y*10.0),
		  XmNmaximum,     temp + 100,
		  NULL);
}

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * drawingarea widget expose callback
 */
void CbExpose(Widget w, XtPointer clientData, XmDrawingAreaCallbackStruct *cbs)
{
  /* ensure last expose event in sequence */
  if (((XExposeEvent *)cbs->event)->count == 0)
    Refresh();
}


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * plottingarea widget expose callback
 */
void CbExposePlot(Widget w, XtPointer clientData, 
		  XmDrawingAreaCallbackStruct *cbs)
{
  /* ensure last expose event in sequence */
  if (((XExposeEvent *)cbs->event)->count == 0)
    RefreshPlot(); /*also refreshes the range area*/
}

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/
/* Moves and resizes modify area selection if the dialog is open and calls 
   another function if the event is 3rd mouse button click.*/

void CbSelectBox(Widget wid, XEvent *event, String *args, int *num_args)
{
  double x,y,sx,sy;
  int xpix,ypix;
  static VecD startp;
  char buff[MAXLINE];
  XButtonEvent *bevent = (XButtonEvent *) event;
  x = bevent->x;
  y = bevent->y;
  sx=wDrawSize.x/zoom.x;
  sy=wDrawSize.y/zoom.y;
  xpix=bevent->x;
  ypix=bevent->y;
  x=(double)( xpix-wDrawSize.x/2.0)/sx+center.x;
  y=(double)(-ypix+wDrawSize.y/2.0)/sy+center.y;
  

  if (*num_args != 1)
    XtError("CbSelectBox error!\n");

  /* start dragging the mouse with left mouse button pressed*/
  if ((strcmp(args[0],"down")==0) && wWinModFlag) {
    modarea.center.x=x;
      modarea.center.y=y;
      modarea.size.x=0.0;
      modarea.size.y=0.0;
      modarea.angle=0.0;
      startp.x=x;
      startp.y=y;
      Redraw();
  }
  /* stop dragging, area selection complete*/
  else if ((strcmp(args[0],"up")==0) && wWinModFlag) {
    
    sprintf(buff, "%g", modarea.center.x);
    XmTextSetString(c[MODAREA_X].text, buff);
    sprintf(buff, "%g", modarea.center.y);
    XmTextSetString(c[MODAREA_Y].text, buff);
    sprintf(buff, "%g", modarea.size.x);
    XmTextSetString(c[MODAREA_SX].text, buff);
    sprintf(buff, "%g", modarea.size.y);
    XmTextSetString(c[MODAREA_SY].text, buff);
    sprintf(buff, "%g", modarea.angle);
    XmTextSetString(c[MODAREA_ROT].text, buff);
    Redraw();
  }
  /* define selection by dragging the mouse */
  else if ((strcmp(args[0],"motion")==0) && wWinModFlag) {  
    if (x >= startp.x) {
      modarea.center.x = x-(x-startp.x)/2;
      modarea.size.x   = (x-startp.x);
    }
    else {
      modarea.center.x = x+(startp.x-x)/2;
      modarea.size.x   = (startp.x-x);
    }
    if (y <= startp.y) {
      modarea.center.y = y-(y-startp.y)/2;
      modarea.size.y   = -(y-startp.y);
    }
    else {
      modarea.center.y = y+(startp.y-y)/2;
      modarea.size.y   = -(startp.y-y);
    }
    Redraw();
  }
  /* move selection */
  else if ((strcmp(args[0],"move")==0) && wWinModFlag) {
    modarea.center.x = x;
    modarea.center.y = y;
    Redraw();
  }
  else if ((strcmp(args[0],"endmove")==0) && wWinModFlag) {
    modarea.center.x = x;
    modarea.center.y = y;
    sprintf(buff, "%g", modarea.center.x);
    XmTextSetString(c[MODAREA_X].text, buff);
    sprintf(buff, "%g", modarea.center.y);
    XmTextSetString(c[MODAREA_Y].text, buff);
    Redraw();
  }
  /* resize selection*/
  else if ((strcmp(args[0],"sresize")==0) && wWinModFlag) {
    startp.x=x;
    startp.y=y;
  }
  else if ((strcmp(args[0],"resize")==0) && wWinModFlag) {
    /*modarea.size.x = fabs(x-modarea.center.x)*2;
    modarea.size.y = fabs(y-modarea.center.y)*2;*/
    /*dist.x=x-modarea.center.x;
    dist.y=y-modarea.center.y;
    if (fabs(dist.x)>SMALLDOUBLE) { 
      modarea.size.x = VLen(dist)*cos(atan(dist.y/dist.x));
      modarea.size.y = VLen(dist)*sin(atan(dist.y/dist.x));
    }
    else {
    }*/
    modarea.size.x = modarea.size.x + (x-startp.x);
    modarea.size.y = modarea.size.y + (y-startp.y);
    
    startp.x=x;
    startp.y=y;
    if (modarea.size.x < 0.0) {
      modarea.size.x = 0;
    }
    if (modarea.size.y < 0.0) {
      modarea.size.y = 0;
    }
    Redraw();
  }
  else if ((strcmp(args[0],"endresize")==0) && wWinModFlag) {
    /*modarea.size.x = fabs(x-modarea.center.x)*2;
    modarea.size.y = fabs(y-modarea.center.y)*2;*/
    sprintf(buff, "%g", modarea.size.x);
    XmTextSetString(c[MODAREA_SX].text, buff);
    sprintf(buff, "%g", modarea.size.y);
    XmTextSetString(c[MODAREA_SY].text, buff);
    Redraw();
  }
  /* rotate selection */
  else if ((strcmp(args[0],"srotate")==0) && wWinModFlag) {
    startp.x=x;
    startp.y=y;
  }
  else if ((strcmp(args[0],"rotate")==0) && wWinModFlag) {
        
    modarea.angle = modarea.angle - (x-startp.x)*10.0;
    
    startp.x=x;
    
    if (modarea.angle < -360.0) {
      modarea.angle = 0;
    }
    if (modarea.angle > 360.0) {
      modarea.angle = 0;
    }
    Redraw(); 
  }
  else if ((strcmp(args[0],"endrotate")==0) && wWinModFlag) {
    
    sprintf(buff, "%g", modarea.angle);
    XmTextSetString(c[MODAREA_ROT].text, buff);
    Redraw(); 
  }
  /* scroll using ctrl+right mouse button*/ /*partially implemented*/
  else if ((strcmp(args[0],"sscroll")==0)) {
    startp.x=x;
    startp.y=y;
  }
  else if ((strcmp(args[0],"scroll")==0)) {
    
    center.x = center.x - (x-startp.x)*0.65;
    center.y = center.y - (y-startp.y)*0.65;
    
    startp.x=x;
    startp.y=y;
    UpdateScrollBars();
    Redraw();
  }
  /* zoom using ctrl+right mouse button*/
  else if ((strcmp(args[0],"szoom")==0)) {
    startp.x=x;
    startp.y=y;
  }
  else if ((strcmp(args[0],"zoom")==0)) {
    
    zoom.y = zoom.y + (y-startp.y)*1.3;
    zoom.x=zoom.y*wDrawSize.x/wDrawSize.y;

    if (zoom.y < 1.0) zoom.y =1.0; 
    if (zoom.y > 200.0) zoom.y = 200.0;

    startp.x=x;
    startp.y=y;
    
    Redraw();
  }
  else if ((strcmp(args[0],"endzoom")==0)) {
    XtVaSetValues(wZoomScale,XmNvalue,(int)zoom.y,NULL);
    UpdateScrollBars();
  }
  /* print atom info */
  else if (strcmp(args[0],"info")==0)  
    /*do not alter selection, print atom info (mouse button 3)*/
    CbPrintAtomInfo(bevent->x,bevent->y);
}




/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*
 * Drawing area mouse click callback:
 * print the nearest atom information.
 */
void CbPrintAtomInfo(double xpix, double ypix/*Widget w, XtPointer clientData, 
		  XmDrawingAreaCallbackStruct *cbs*/)
{
  int /*xpix,ypix,*/j,c2,k,offset;
  /*int c;*not in use*/
  double x,y,sx,sy,dr,drmin,dx,dy;
  static char *aname[5]={"Bf","Af","??","A","B"};
  VecD rs,invWid,xv,iof[] = {{0,0},{1,0},{1,1},{0,1},{-1,1}};
  VecI cc,cc2;

  if (/*cbs->event->xany.type==ButtonPress*/1) {

    sx=wDrawSize.x/zoom.x;
    sy=wDrawSize.y/zoom.y;
    /*xpix=cbs->event->xbutton.x;
    ypix=cbs->event->xbutton.y;*/
    x=( xpix-wDrawSize.x/2.0)/sx+center.x;
    y=(-ypix+wDrawSize.y/2.0)/sy+center.y;
    xv.x=x;
    xv.y=y;

    /* Get the linear index of the current cell to c */
    VDiv(invWid, cells, simArea);
    VSAdd(rs,xv,0.5,simArea);
    VMul(cc,rs,invWid);
    /*c=VLinear(cc,cells)+nAtomOrig;*not in use*/
    drmin=1.0e20;
    k=-1;
    /* Go through atoms in the current and neighboring cells */
    for (offset = 0; offset < 5; offset ++) {
      VAdd(cc2,cc,iof[offset]);
      if (cc2.x>=0 && cc2.x<cells.x && cc2.y>=0 && cc2.y<cells.y) {
	c2=VLinear(cc2,cells)+nAtomOrig;
	for (j=cellList[c2];j>=0;j=cellList[j]) {
	  dx=r[j].x-xv.x;
	  dy=r[j].y-xv.y;
	  dr=(dx*dx+dy*dy);
	  if (dr<drmin) {
	    k=j;
	    drmin=dr;
	  }
	}
      }
    }
    fprintf(stderr,"Pos %g %g ",x,y);
    if (k>=0) {
      fprintf(stderr," atom %d (%s) at %g %g v %g %g Epot %g",
	      k,aname[atype[k]+2],r[k].x,r[k].y,rv[k].x,rv[k].y,potEn[k]);
      if (sinfo==SXX || sinfo==SXY || sinfo==SYY) 
	fprintf(stderr," stress %g",stressij[k]);
    }
    fprintf(stderr,"\n");
  }
}

/*--------------------------------------------------------------------*/
/*--------------------------------------------------------------------*/

/*needed by the notorius tracking dialog*/
void CbSaveTra(Widget wTextSave,XtPointer client_data,XtPointer call_data)
{
  int i;
  char *tulostus = XmTextFieldGetString(wTextSave);

  for(i=0; i < 5; i++){
    numero[i] = -1;
  }
 
  sscanf(tulostus,"%d,%d,%d,%d,%d",&numero[0],&numero[1],
	 &numero[2],&numero[3],&numero[4]); 
  printf(" %d,%d,%d,%d,%d \n",numero[0],numero[1],numero[2],
	 numero[3],numero[4]);
  
}

/*--------------------------------------------------------------------*/
/*--------------------------------------------------------------------*/
/*this function is called everytime a dialog is closed either by the window
  manager or by the application*/
void CbCloseDialog(Widget widget, XtPointer client_data, XtPointer call_data) 
{
  Widget reset_widget = (Widget) client_data;
  XtSetSensitive(reset_widget, True);
  if (widget != NULL) {
    if ((wSeamMan != NULL) && (widget == XtParent(wSeamMan)))
      wWinSeamFlag = False;
    else if ((wParamMan != NULL) && (widget == XtParent(wParamMan)))  {
      if(wWinSeamFlag) XtPopdown(XtParent(wSeamMan));
      strainFlag = False;
      CalcAtoms();
      InitRun();
      XtSetSensitive(wRunPushB, True);
      Redraw();
      RedrawPlot(1);
    }
    else if ((wTrackMan != NULL) && (widget == XtParent(wTrackMan)))
      wWinTrackFlag=0;
    else if ((wAniMan != NULL) && (widget == XtParent(wAniMan)))
      wWinAniFlag=0;
    else if ((wModMan != NULL) && (widget == XtParent(wModMan))) {
      wWinModFlag= False;
      Redraw();
    }
    
  }
}

/*--------------------------------------------------------------------*/
/*--------------------------------------------------------------------*/

/*callback function for all menus in menubar or display menu*/
void CbMenu(Widget menu_item,XtPointer client_data,XtPointer call_data)
{
  switch ((int) client_data) {
  case M_NEW:  /*new simulation*/
    wWinSeamFlag = 0;
    if (runningFlag) { /* stop a running simulation */
      XtRemoveWorkProc(id);
      runningFlag = False;
      XtVaSetValues(wRunPushB, XmNalignment,XmALIGNMENT_CENTER, 
		    XtVaTypedArg, XmNlabelString, T_STRING("Run"),
		    NULL);
    }
    if (sinfo == SVECT) {
      sinfo = STYPE;
    }
    if (savetscheme)
      fclose(tsavefile);
    cool.programon = 0;
    ani.snaphelp  = 0;
    XtSetSensitive(wRunPushB, False);
    SaveParams(STORE);
    CalcAtoms();
    OpenDialog(wParamMan,menu_item,"param");
    XtSetSensitive(wPushBCancel, True);
    Redraw();
    RedrawPlot(0);
    break;
  case M_SNAPSHOT:   /*save snapshot*/
    SnapShot();
    break;
  case M_ANIMATION:   /*save animation*/
    wWinAniFlag = True;
    if (runningFlag) { /* stop a running simulation*/
      XtRemoveWorkProc(id);
      runningFlag = False;
      /*XtSetSensitive(wStrainPushB, False);*/
      XtVaSetValues(wRunPushB,  XmNalignment,XmALIGNMENT_CENTER,
		    XtVaTypedArg,XmNlabelString, T_STRING("Run"),
		    NULL);
    }
    /*if (vectorFlag) {
      vectorFlag = False;*/
    /*XtVaSetValues(wVectorToggle, XmNset, False, NULL);
      }*/
    OpenDialog(wAniMan, menu_item,"ani");
    wWinAniFlag = True;
    Redraw();
    break;
  case M_EXIT:  /*bexit*/
    bexit(0);
    break;

  case M_STRAIN: /*strain and pressure*/ 
    OpenDialog(wStrainMan, menu_item,"strain");
    /*Redraw();*/
    break;
  case M_TEMPERATURE: /*temperature*/
    OpenDialog(wTempMan, menu_item,"temp");
    /*Redraw();*/
    break;
  case M_NEWCRACK: /*make new crack*/
    /*CreateInitialCrack(r,&nAtom,crack.center.x,crack.center.y,
		       crack.size.x,crack.size.y,crack.angle);*/
    /*modinfo=M_MODRM;*/
    /*OpenModDialog(menu_item,client_data,call_data);*/
    OpenDialog(wModMan,menu_item,"mod");
    wWinModFlag = True;
    Redraw();
    break;
  case M_RESCALE:
    plotdata.rescale=1;
    break;
  case M_ZOOM: /*zooming*/ 
    OpenDialog(wZoomMan, menu_item,"zoom");
    break;
  case M_COLOR: /*color ranges*/ 
    OpenDialog(wColorMan, menu_item,"color");
    break;

  case M_TRACK:
    if (sinfo == SVECT) {
      sinfo = SKIN;
      /*XtVaSetValues(wVectorToggle, XmNset, False, NULL);*/
    }
    OpenDialog(wTrackMan, menu_item,"track");
    wWinTrackFlag = True;
    /*Redraw();*/
    break;

    /*below this are display -option menu items*/

  case M_KIN:
    TASensitive(AVRCUT, False);
    sinfo = SKIN;
    Redraw();
    break;
  case M_POT:
    TASensitive(AVRCUT, False);
    sinfo = SPOT;
    Redraw();
    break;
  case M_SIGMAXX:
    TASensitive(AVRCUT, False);
    sinfo = SXX;
    Redraw();
    break;
  case M_SIGMAXY:
    TASensitive(AVRCUT, False);
    sinfo = SXY;
    Redraw();
    break;
  case M_SIGMAYY:
    TASensitive(AVRCUT, False);
    sinfo = SYY;
    Redraw();
    break;
  case M_ATOMTYPE:
    TASensitive(AVRCUT, False);
    sinfo = STYPE;
    Redraw();
    break;
  case M_FORCE:
    sinfo = SFORCE;
    TASensitive(AVRCUT, False);
    Redraw();
    break;  
  case M_VECTORS:
    sinfo = SVECT;
    TASensitive(AVRCUT, True);
    Redraw();
    break; 
  case M_MODRM:
    modinfo=M_MODRM;
    TASensitive(MODVX, False);
    TASensitive(MODVY, False);
    break;
  case M_MODSTOP:
    modinfo=M_MODSTOP;
    TASensitive(MODVX, False);
    TASensitive(MODVY, False);
    break;
  case M_MODTYPE1:
    modinfo=M_MODTYPE1;
    TASensitive(MODVX, False);
    TASensitive(MODVY, False);
    break;
  case M_MODTYPE2:
    modinfo=M_MODTYPE2;
    TASensitive(MODVX, False);
    TASensitive(MODVY, False);
    break;
  case M_MODVEL:
    modinfo=M_MODVEL;
    TASensitive(MODVX, True);
    TASensitive(MODVY, True);
    break;  
  case M_MODFIX:
    modinfo=M_MODFIX;
    TASensitive(MODVX, True);
    TASensitive(MODVY, True);
    break;  
  case M_MODRMF:
    modinfo=M_MODRMF;
    TASensitive(MODVX, False);
    TASensitive(MODVY, False);
    break;  
  case M_MODFORCE:
    modinfo=M_MODFORCE;
    TASensitive(MODVX, True);
    TASensitive(MODVY, True);
  }
}

/*--------------------------------------------------------------------*/
/*--------------------------------------------------------------------*/

void CbList(Widget menu_item,XtPointer client_data,XtPointer call_data)
{
  XmListCallbackStruct *cbs = (XmListCallbackStruct *) call_data;
  char *choice;
  
  XmStringGetLtoR(cbs->item, XmFONTLIST_DEFAULT_TAG,&choice);
  /*fprintf(stderr,"item: %s\n",choice);*/
  strcpy(tfname,choice);
}
