#include <X11/Intrinsic.h>

/***************************************************************************/
/* macro definitions */


/*general*/
#define MAXLINE 250
#define MAXLABELS 30
#define HORIZONTAL 0
#define VERTICAL   1
#define SMALLDOUBLE 0.000000001
#define PRI(x) fprintf(stderr,"x %d\n",x);
#define PR(x) fprintf(stderr,x);fprintf(stderr,"\n");
#define ABS(x)  ((x)>=0 ? (x):-(x))
#define SQ(x) ((x) * (x))
#define SQ4(x) (SQ(x)*SQ(x))
#define T_STRING(s) XmRString, s, strlen (s) + 1
#define ALLOC(y, x, t) y = (t *) realloc (y, (x) * sizeof (t))

/* vector operations */
#define VSet(v, sx, sy) v.x = sx, v.y = sy
#define VZero(v) VSet (v, 0, 0)
#define VSCopy(v2, s1, v1) v2.x = (s1) * v1.x, v2.y = (s1) * v1.y
#define VScale(v, s) VSCopy (v, s, v)
#define VAdd(v1, v2, v3) v1.x = v2.x + v3.x, v1.y = v2.y + v3.y
#define VSub(v1, v2, v3) v1.x = v2.x - v3.x, v1.y = v2.y - v3.y
#define VMul(v1, v2, v3) v1.x = v2.x * v3.x, v1.y = v2.y * v3.y
#define VDiv(v1, v2, v3) v1.x = v2.x / v3.x, v1.y = v2.y / v3.y
#define VSAdd(v1, v2, s3, v3) \
   v1.x = v2.x + (s3) * v3.x, v1.y = v2.y + (s3) * v3.y
#define VVSAdd(v1, s2, v2) VSAdd (v1, v1, s2, v2)
#define VProd(v) (v.x * v.y)
#define VLinear(p, s) p.y * s.x + p.x
#define VLenSq(v) (SQ (v.x) + SQ (v.y))
#define VLen(v) (sqrt(VLenSq(v)))

/* The following are used in connection with array 'borderatom' */
#define BORDER_LEFT   1
#define BORDER_RIGHT  4
#define BORDER_BOTTOM 8
#define BORDER_TOP    16

/* units and defaults  */
#define TEMPERATURE 0.0 /* initial temperature of the system */
#define UNITTIME 1.0e-12  /* in seconds */
#define UNITDIST 3.68e-10 /* in meters, copper lattice const., */ 
			     /*acc. to Handbook of Chem. and Phys.*/
#define EPSILONCOPPER 1.260183e-19 /* in joules, acc. to D. Greenspan */
#define MASSCOPPER 1.055207e-25 /*in kg, acc. to Handbook of Chem. and Phys.*/
#define UNITENERGY 9.174081 /* MASSCOPER * sqr(UNITDIST/UNITTIME) */
#define UNITTEMP 994.8816 /* Kb/UNITENERGY */

/* Definitions for text widget's modify verification */
#define ALLOW_NUMBERS 1
#define ALLOW_MINUS 2
#define ALLOW_DOT 4

/* for text arrow widgets (text-arrow numbers,dot,minus)*/
#define TAN   0
#define TAND  1
#define TANMD 2
#define TANM  3

/* Callback definitions*/

/* Buttons */
#define BOUNDARY          0
#define CRACK             1
#define RUN               2
#define STRAIN            3
#define PARAM_OK          4
#define CANCEL            5
#define FIXED_TOGGLE      6
#define TSCALE_TOGGLE     7
#define VERTIC            8
#define START_COOL        9
#define CLOSE_TEMP       10
#define SUGGEST          11 
#define SEAM_OK          12
#define SEAM             13
#define CLOSE_ANI        14
#define SET_ANI          15
#define LIM_TOGGLE       16
#define ENDEND           17
#define QUENCH_TOGGLE    18
#define RANDOMIZE        19
#define SETTRACKTOG      20 /* Other Tracking pointer-*/
#define CLOSE_TRK        21
#define SAVETRK          22
#define SHOWTRK          23
#define PERX             24     /*periodicX*/
#define PERY             25
#define CLOSE_STRAIN     26
#define CLOSE_ZOOM       27
#define CLOSE_MOD        28
#define MODIFY           29
#define CLOSE_COLOR      30
#define COLRANUPD        31
#define COLRANHIDE       32
#define MODSHOWSEL       33
#define TAPPLY           34
#define TADJUST          35
#define TSAVE            36
#define TADJUST2         37

/* Scales */
#define DIAM 0
#define UPDATE 1
#define PLOTSCALE 2    /*used in plot drawing*/
#define ZOOMSCALE 3

/* Menu items */
#define M_NEW          0
#define M_SNAPSHOT     1
#define M_ANIMATION    2
#define M_EXIT         3
#define M_STRAIN       4
#define M_TEMPERATURE  5
#define M_NEWCRACK     6
#define M_TRACK        7
#define M_RESCALE      8
#define M_KIN          9
#define M_POT         10
#define M_SIGMAXX     11
#define M_SIGMAXY     12
#define M_SIGMAYY     13
#define M_ATOMTYPE    14
#define M_VECTORS     15
#define M_ZOOM        16
#define M_COLOR       17
/*definitions for area modifier*/
#define M_MODRM       18
#define M_MODSTOP     19
#define M_MODTYPE1    20
#define M_MODTYPE2    21
#define M_MODVEL      22
#define M_MODFIX      23
#define M_MODRMF      24
#define M_MODFORCE    25
#define M_FORCE       26

/* 
 *  Text widgets: The order of these must agree with the container array 
 *  in graphvar.h 
 *  
 *  NOTE: These are now reordered so that in CbTextValueChanged() it is easy
 *  to decide whether to call CalcAtoms() or not after value change. AK  
 */

/* These need a call to CalcAtoms() */

#define SIZE_X         0
#define SIZE_Y         1
#define ROT_L          2
#define ROT_R          3
#define DISP_X         4
#define DISP_Y         5
#define CRACK_X        6
#define CRACK_Y        7
#define CRACK_SX       8
#define CRACK_SY       9
#define CRACK_ROT     10
#define L_PARAM       11
#define RCUT          12
#define EPSILON       13
#define BORDER        14
#define BND_D         15
#define EPSILON2      16
#define L_PARAM2      17
#define RELAXATION    18
#define AMOVE         19
#define BMOVE         20
#define DOWNCUT       21
#define UPCUT         22
#define DOWNCUTA      23
#define UPCUTA        24
#define DOT           25

/* These do not need a call to CalcAtoms() */

#define STRAIN_P      26
#define STRAIN_T      27
#define PRESSURE      28
#define ZOOM_X        29
#define ZOOM_Y        30
#define AVRCUT        31
#define TEM           32
#define STARTTIME      33
#define SLOPE         34
#define COOLTEMP       35
#define WAIT          36
#define BEGINAT       37
#define ENDAT         38
/*----Track container -*/
#define TRANGE        39
#define TFACTOR       40
#define TBFACTOR      41
/*---------------------*/
#define MODAREA_X     42
#define MODAREA_Y     43
#define MODAREA_SX    44
#define MODAREA_SY    45
#define MODAREA_ROT   46
#define MODVX         47
#define MODVY         48
#define COLRANKMIN    49
#define COLRANKMAX    50
#define COLRANPMIN    51
#define COLRANPMAX    52
#define COLRANSMIN    53
#define COLRANSMAX    54
#define COLRANFMIN    55
#define COLRANFMAX    56



/* definitions for show potential/kinetic/vectors etc*/
#define SKIN   0
#define SPOT   1
#define SXX    2
#define SXY    3
#define SYY    4
#define STYPE  5
#define SVECT  6
#define SFORCE 7

/***************************************************************************/

/*structs*/
typedef struct {double x, y;} VecD;
typedef struct {int x, y;} VecI;
typedef struct {int x, y, z;} VecI3;
typedef struct {VecD x, y;} MtrxD;

/* resources to allow user specification at runtime */
typedef struct {
  int wSize;
  int bpp;
  long int icix0,iciy0;
  Pixel color[5];
} AppData;

/*a struct needed by ROWParams, rowi and rowd*/
typedef struct {
  int writingmode;
  char temp[MAXLINE];
  int ivar;
  double dvar;
  FILE *pfile;
} ROWPstruct;


typedef struct {
  VecD center;
  VecD size;
  double angle;
} RArea;

typedef struct {
  double xscale;             /*plot time scale*/
  double yscalingratio;      /*how much the plot is scaled when it doesn't 
			       fit the current range*/
  int rescale;               /*rescale plot in the current cycle*/
} Plotdata;

typedef struct {
  int aniFlag;      /* snapshot animation recording on/off*/
  int limitsFlag;   /* limit number of animation shots on/off*/
  double beginat;  /* Used in Animation().if limitsFlag then animation 
		      starts from here*/
  double endat;   /* Used in Animation().if limitsFlag then animation 
		     ends to here*/
  double waittime;  /* Used in animation. Time between two snapshots.*/
  double snaphelp;    /* Used in Animation() in time checking.*/
  int endendFlag;   /* end program when animation done*/
} AniData;
  
typedef struct {
  double cooltemp;
  double starttime;
  double tempvalue;
  double slope;
  int QuenchFlag;/* AK: if true do quenching (see LeapfrogIntegrate())*/
  int TScaleFlag;
  int programon;
} CoolData;

typedef struct {
  VecD disp;
  double Amove;
  double Bmove;
  double dot; 
  double upcut;
  double downcut;
  double downcuta;
  double upcuta;
  double boundpos;                  /* place of the boundary        */
  double rotL;                      /* rotation of the two lattices */
  double rotR;  
} Seamdata;

typedef struct {
  int Update;
  int hide;
  double EkMin;
  double EkMax;
  double EpMin;
  double EpMax;
  double SMin;
  double SMax;
  double FMin;
  double FMax;
} ColorRanges;  

/* A struct for text widgets */
typedef struct {
  Widget text;
  Widget arrow1;
  Widget arrow2;
  int length;
  int nmdvalue;
  double *value;
  double min;
  double max;
  double step;
} Container;


/***************************************************************************/

/*function prototypes */
int main(int, char**);
void CatchHUP();
void SetDefaults();
void CalcSimArea();
void rowd(char*,double*,ROWPstruct);
void rowi(char*,int*,ROWPstruct);
void ROWParams(char*,char*);
void readTscheme(char*);
void checkTscheme();
void getsomearguments(int, char**);
long int getseed();
void readatomsfromfile();
double nint(double); 
void bexit(int);
