#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fitsigma.h"
#include "control.h"




/*-------------------------------------------------------*/
/*-------------------------------------------------------*/

/*used in potential calculations*/

#define AX 0.7
#define BX 0.95
#define CX 1.3

double fitsigma(double r0, double rc)
{
  double Emin;
  /*definitions*/

  double ax,bx,cx,tol=1.0e-15;
  double sigma;
  
  sigma=pow(0.5*(1.0-pow(r0/rc,7.0))/(1.0-pow(r0/rc,13.0)),1.0/6.0)*r0;
  ax=AX*sigma;
  bx=BX*sigma;
  cx=CX*sigma;
  Emin=golden(ax,bx,cx,tol,&sigma,r0,rc);
  return sigma;
}

#undef AX 
#undef BX 
#undef CX 

/*-------------------------------------------------------*/
/*-------------------------------------------------------*/

/* 
   System potential energy.
*/

double E(double sigma, double r0, double rc)
{
  int i,imin,imax;
  int j,jmin,jmax;
  double r,etmp;
  double rs12,rs6,rs1,rs0;
  double x,ir,phip,phi;
  static double eps=1.0;

  imax=jmax=1.3*rc/r0;
  imin=jmin=-imax;

  rs12=rs6=rs1=rs0=0.0;
  etmp=0.0;

  for (i=imin;i<=imax;i++) {
    for (j=jmin;j<=jmax;j++) {
      if (i==0 && j==0) continue;
      ir=i*i+i*j+j*j;
      r=r0*sqrt(ir);
      if (r>rc) continue;
      x=sigma/r;
      x=x*x*x*x*x*x;
      rs12+=x*x;
      rs6+=x;
      rs1+=r;
      rs0+=1.0;
    }
  }
  
  x=sigma/rc; 
  x=x*x*x*x*x*x;    
  phip=-12.0*eps/rc*(x*x-0.5*x);
  phi=eps*(x*x-x);
  
  etmp=eps*(rs12-rs6)+phip*(rc*rs0-rs1)-rs0*phi;
  return etmp;
}


/*-------------------------------------------------------*/
/*-------------------------------------------------------*/

/*auxiliary function for fitsigma*/

#define R 0.61803399
#define C (1.0-R)
#define SHFT2(a,b,c) (a)=(b);(b)=(c);
#define SHFT3(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);

double golden(double ax, double bx, double cx, double tol, double *xmin,
	      double r0, double rc)
{
  double f1,f2,x0,x1,x2,x3;
  
  x0=ax;
  x3=cx;
  if (fabs(cx-bx) > fabs(bx-ax)) {
    x1=bx;
    x2=bx+C*(cx-bx);
  } 
  else {
    x2=bx;
    x1=bx-C*(bx-ax);
  }
  f1=E(x1,r0,rc);
  f2=E(x2,r0,rc);
  while (fabs(x3-x0) > tol*(fabs(x1)+fabs(x2))) {
    if (f2 < f1) {
      SHFT3(x0,x1,x2,R*x1+C*x3);
      SHFT2(f1,f2,E(x2,r0,rc));
    } 
    else {
      SHFT3(x3,x2,x1,R*x2+C*x0);
      SHFT2(f2,f1,E(x1,r0,rc));
    }
  }
  if (f1 < f2) {
    *xmin=x1;
    return f1;
  } 
  else {
    *xmin=x2;
    return f2;
  }
}

#undef C
#undef R
#undef SHFT2
#undef SHFT3
