

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

/* motif libraries */
#include <Xm/ArrowB.h>   
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/MwmUtil.h>
#include <Xm/MainW.h>
#include <Xm/DialogS.h>
#include <Xm/PanedW.h>
#include <Xm/CascadeB.h>
#include <Xm/SeparatoG.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeBG.h>
#include <Xm/ToggleBG.h>
#include <Xm/ToggleB.h>
#include <Xm/List.h>

#include "control.h"
#include "graphics.h"
#include "callback.h"
#include "calc.h"
#include "fitsigma.h"
#include "simu.h"
#include "graphvar.h"  /*declare global variables needed in graphics.c*/
#include "gvar.h"      /*load global variables*/


/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/



/*
 * Setup graphics. Creates the main window, initializes the Xt Toolkit 
 * and parses Xt command line options. Called in main().
 */
void SetupGraphics(int argc, char **argv)
{
  Widget wForm, wForm2, wFrame, wRowCol, wRowCol2,wRowCol3;
  Widget wLabel, wScrolledW,wScale;
  int scrn;
  Widget wMenuBar,wDisplayMenu,wWinMain;
  Widget wFileMenu,wControlsMenu,wViewMenu;
  String fallbacks[]= {
    "Boundary*background: lightgrey",
    "Boundary*foreground: black",
    "Boundary*title: Boundary 2D",
    NULL
  };
  /*translation table for main drawarea to get mouse drag callbacks etc*/
/*  String translations =
    "<Btn1Down>:   CbSelectBox(down)   ManagerGadgetArm() \n\
     <Btn1Up>:     CbSelectBox(up)     ManagerGadgetActivate() \n\
     <Btn3Up>:     CbSelectBox(up3)    ManagerGadgetActivate() \n\
     <Btn1Motion>: CbSelectBox(motion) ManagerGadgetButtonMotion()";
*/
 String translations =
   "Ctrl Shift<Btn1Down>:   CbSelectBox(down)      ManagerGadgetArm() \n\
    Ctrl Shift<Btn1Up>:     CbSelectBox(up)        ManagerGadgetActivate() \n\
    Ctrl Shift<Btn1Motion>: CbSelectBox(motion)    ManagerGadgetButtonMotion() \n\
    ~Ctrl~Shift<Btn1Down>:  CbSelectBox(move)      ManagerGadgetArm() \n\
    ~Ctrl~Shift<Btn1Up>:    CbSelectBox(endmove)   ManagerGadgetActivate() \n\
    ~Ctrl~Shift<Btn1Motion>:CbSelectBox(move)      ManagerGadgetButtonMotion() \n\
    Ctrl~Shift<Btn1Down>:   CbSelectBox(sresize)   ManagerGadgetArm() \n\
    Ctrl~Shift<Btn1Up>:     CbSelectBox(endresize) ManagerGadgetActivate() \n\
    Ctrl~Shift<Btn1Motion>: CbSelectBox(resize)    ManagerGadgetButtonMotion() \n\
    ~Ctrl Shift<Btn1Down>:  CbSelectBox(srotate)   ManagerGadgetArm() \n\
    ~Ctrl Shift<Btn1Up>:    CbSelectBox(endrotate) ManagerGadgetActivate() \n\
    ~Ctrl Shift<Btn1Motion>:CbSelectBox(rotate)    ManagerGadgetButtonMotion() \n\
    ~Ctrl~Shift<Btn3Down>:  CbSelectBox(sscroll)   ManagerGadgetArm() \n\
    ~Ctrl~Shift<Btn3Up>:    CbSelectBox(endscroll) ManagerGadgetActivate() \n\
    ~Ctrl~Shift<Btn3Motion>:CbSelectBox(scroll)    ManagerGadgetButtonMotion() \n\
    Ctrl~Shift<Btn3Down>:   CbSelectBox(szoom)     ManagerGadgetArm() \n\
    Ctrl~Shift<Btn3Up>:     CbSelectBox(endzoom)   ManagerGadgetActivate() \n\
    Ctrl~Shift<Btn3Motion>: CbSelectBox(zoom)      ManagerGadgetButtonMotion() \n\
    Ctrl Shift<Btn3Up>:     CbSelectBox(info)      ManagerGadgetActivate()";


  XtActionsRec actions;
  
  /* initialize graphics, toplevel shell widget  */
  
  wTop = XtVaAppInitialize(&app, "Boundary"/*Xmd*/, NULL, 0,
			   &argc, argv, fallbacks,NULL,NULL);

  XtGetApplicationResources(wTop, NULL, NULL, 0, NULL, 0);/*Xt cmd line opts*/
  
  XtAddEventHandler(wTop,StructureNotifyMask,False,  
		    (XtEventHandler) SizeMaster,NULL);/*disable resize*/
  
  SetupColors(16); 
  
  /* Boundary main window */
  wWinMain = XtVaCreateManagedWidget ("main window",
    xmMainWindowWidgetClass, wTop,
    NULL);
   
  wPlotSize.x=wDrawSize.x;
  wRangeSize.x=wPlotSize.x;
  wRangeSize.y=15;
  wPlotSize.y=(int)(YPLOTFRACTION*wDrawSize.y)-wRangeSize.y;
  
  actions.string = "CbSelectBox";
  actions.proc = CbSelectBox;
  XtAppAddActions(app,&actions,1);
 
  /*Menubar*/
  wMenuBar = XmCreateMenuBar (wWinMain, "menubar",NULL,0);
  wFileMenu=CreateMenu(wMenuBar,XmMENU_PULLDOWN,"File",'F',filemenu);
  wControlsMenu=CreateMenu(wMenuBar,XmMENU_PULLDOWN,"Controls",'C',
			   controlsmenu);
  wViewMenu=CreateMenu(wMenuBar,XmMENU_PULLDOWN,"View",'V',viewmenu);
  XtManageChild(wMenuBar);
  
  /* form and rowcol widgets for the whole window */
  wForm = CreateForm(wWinMain,5);
  wRowCol = CreateMainRowCol(wForm);

  /* text display */
  wTextInfo = CreateTextInfo(wRowCol,15,13,3,2);
  
  /* Zoom scale */
  wZoomScale = CreateScale(wRowCol,1,200,(int)(zoom.y),"Zoom",
			   ZOOMSCALE);

  /*display menu*/
  wDisplayMenu=CreateMenu(wRowCol,XmMENU_OPTION,"",'D',displaymenu);
  XtManageChild(wDisplayMenu);
  
  /* Vector averaging */
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Vector averaging");
  wRowCol2 = CreateRCTA(wForm2,wLabel,"",AVRCUT);

  /* Atom diam scale */
  wScale = CreateScale(wRowCol,10,200,(int)(100.0*diam),"Atom diam (%)",
			    DIAM);
  /* Update step */
  wScale = CreateScale(wRowCol,(int)(1000*deltaT),400,updateStep,
			    "Update step",UPDATE);
  /* Plot scale */
  wScale = CreateScale(wRowCol,1,10,(int)(plotdata.xscale),"Plot time scale",
			   PLOTSCALE);
  /* run/pause button */
  wRunPushB = CreatePushB(wRowCol,"Run",RUN);

  /* scrolled window */
  
  wFrame = XtVaCreateManagedWidget("Frame",
    xmFrameWidgetClass,        wForm,
    XmNtopAttachment,          XmATTACH_FORM,
    XmNleftAttachment,         XmATTACH_WIDGET,
    XmNleftWidget,             wRowCol,
    XmNbottomAttachment,       XmATTACH_FORM,
    XmNshadowType,             XmSHADOW_IN,
    NULL);
 
  wRowCol3=XtVaCreateManagedWidget("plotwidget",xmRowColumnWidgetClass, 
				   wFrame,NULL);  	   

  wScrolledW = XtVaCreateManagedWidget("Scrolled Window",
    xmScrolledWindowWidgetClass, wRowCol3,
    XmNtopAttachment,          XmATTACH_FORM,
    XmNleftAttachment,         XmATTACH_FORM,
    XmNrightAttachment,        XmATTACH_FORM,
    XmNbottomAttachment,       XmATTACH_FORM,
    XmNscrollBarDisplayPolicy, XmSTATIC,
    XmNspacing,                0,
    NULL);
  
  wHScrollB = XtVaCreateManagedWidget("Horiz. ScrollBar",
    xmScrollBarWidgetClass,    wScrolledW,
    XmNorientation,            XmHORIZONTAL,
    XmNminimum,                -(int)((simArea.x - zoom.x) / 0.2 + 5),
    XmNmaximum,                (int)((simArea.x - zoom.x) / 0.2 + 105),
    XmNsliderSize,             100,
    XmNincrement,              10,
    NULL);
  XtAddCallback(wHScrollB, XmNvalueChangedCallback, CbScroll,
		(XtPointer)(long)HORIZONTAL);
  XtAddCallback(wHScrollB, XmNdragCallback, CbScroll,
		(XtPointer)(long)HORIZONTAL);
  
  wVScrollB = XtVaCreateManagedWidget("Vertical ScrollBar",
    xmScrollBarWidgetClass,    wScrolledW,
    XmNorientation,            XmVERTICAL,
    XmNprocessingDirection,    XmMAX_ON_TOP,
    XmNminimum,                -(int)((simArea.y - zoom.y) / 0.2 + 5),
    XmNmaximum,                (int)((simArea.y - zoom.y) / 0.2 + 105),
    XmNincrement,              10,
    XmNsliderSize,             100,
    NULL);
  XtAddCallback(wVScrollB, XmNvalueChangedCallback, CbScroll,
		(XtPointer)(long)VERTICAL);
  XtAddCallback(wVScrollB, XmNdragCallback, CbScroll,
		(XtPointer)(long)VERTICAL);

  /* drawing area */

  wDraw = XtVaCreateManagedWidget("DrawingArea",
    xmDrawingAreaWidgetClass,  wScrolledW,
    XmNtranslations,XtParseTranslationTable(translations),
    XmNwidth,                  wDrawSize.x,
    XmNheight,                 wDrawSize.y, 
    NULL);
  XtAddCallback(wDraw, XmNexposeCallback, CbExpose, 0);
  /*XtAddCallback(wDraw, XmNinputCallback,*/ /*CbMouseInput*//*CbSelectBox, 0);
*/  XmScrolledWindowSetAreas(wScrolledW, wHScrollB, wVScrollB, wDraw);

  /* Plot range area */

  wRange = XtVaCreateManagedWidget("RangesArea",
    xmDrawingAreaWidgetClass,  wRowCol3,
    XmNwidth,                  wRangeSize.x,
    XmNheight,                 wRangeSize.y, 
    XmNtopWidget,              wDraw,
    XmNtopAttachment,          XmATTACH_WIDGET,
    XmNscrollBarDisplayPolicy, XmSTATIC,
    XmNspacing,                0,
    NULL);
  XtAddCallback(wRange, XmNexposeCallback, CbExposePlot, 0);

  /* Plotting area */
  
  wPlot = XtVaCreateManagedWidget("PlottingArea",
    xmDrawingAreaWidgetClass,  wRowCol3,
    XmNwidth,                  wPlotSize.x,
    XmNheight,                 wPlotSize.y, 
    XmNtopAttachment,          XmATTACH_WIDGET,
    XmNtopWidget,              wRange,
    XmNscrollBarDisplayPolicy, XmSTATIC,
    XmNspacing,                0,
    NULL);
  XtAddCallback(wPlot, XmNexposeCallback, CbExposePlot, 0);

  /* create graphic context (gc) */
 
  dpy = XtDisplay(wTop);
  scrn = DefaultScreen(dpy);
  gc = XCreateGC(dpy, RootWindow(dpy, scrn), 0, NULL);

  /* create and clear pixmap for offscreen graphics */

  pixmap = XCreatePixmap(dpy, RootWindow(dpy, scrn), wDrawSize.x,
			 wDrawSize.y,
			 DefaultDepth(dpy, scrn));
  XSetForeground(dpy, gc, color[BACKGROUND]);
  XFillRectangle(dpy, pixmap, gc, 0, 0, wDrawSize.x, wDrawSize.y);

  /* Plotting area pixmap */
  rangepixmap = XCreatePixmap(dpy, RootWindow(dpy, scrn), wRangeSize.x,
			 wRangeSize.y,
			 DefaultDepth(dpy, scrn));
  plpixmap = XCreatePixmap(dpy, RootWindow(dpy, scrn), wPlotSize.x,
			 wPlotSize.y,
			 DefaultDepth(dpy, scrn));
  XSetForeground(dpy, gc, color[BACKGROUND]);
  XFillRectangle(dpy, plpixmap, gc, 0, 0, wPlotSize.x, wPlotSize.y);
  XFillRectangle(dpy, rangepixmap, gc, 0, 0, wRangeSize.x, wRangeSize.y);

  XtVaSetValues(wWinMain,XmNmenuBar,wMenuBar,XmNworkWindow,wForm,NULL);
  
  /* realize entire widget tree */
  XtRealizeWidget(wTop);
  
  XtSetSensitive(wRunPushB, False);
  /*XtSetSensitive(c[AVRCUT].text, False);*/
  TASensitive(AVRCUT,False);
}


/*---------------------------------------------------------------*/
/*---------------------------------------------------------------*/


/* Opens a dialog. wManager is the main manager widget of that dialog (usually
   a pane and menu_item is usually a menu item that opens that dialog 
   (sometimes a button). When the dialog is opened for the 1st time and closed,
   it remains in the memory and then it can be opened again just by popping 
   it up which is faster that re-creating all widgets. In addition, unvisible
   but existing widgets can be referred to and their values can be changed.*/

void OpenDialog(Widget wManager, Widget menu_item, char *name)
{
 /* fprintf(stderr,"opendialog ");
  fprintf(stderr,"%s\n",name);*/
  if (wManager != NULL) {
    /*fprintf(stderr,"...branch 1 ");*/
    XtSetSensitive(menu_item, False);
    XtManageChild(wManager);
    XtPopup(XtParent(wManager),XtGrabNone);
    XSync(dpy, False);
  }
  else {
    /*fprintf(stderr,"...branch 2 ");*/
    if (strcmp(name,"mod")==0) { 
      /*fprintf(stderr,"mod\n");*/
      OpenModDialog(menu_item,NULL,NULL);
    }
    else if (strcmp(name,"param")==0) {
      /*fprintf(stderr,"param");*/
      OpenParamDialog(menu_item,NULL,NULL);
    }
    else if(strcmp(name,"seam")==0) {
      /*fprintf(stderr,"seam");*/
      OpenSeamDialog(menu_item,NULL,NULL);
    }
    else if (strcmp(name,"ani")==0) 
      OpenAniDialog(menu_item,NULL,NULL);
    else if (strcmp(name,"strain")==0) 
      OpenStrainDialog(menu_item,NULL,NULL);
    else if (strcmp(name,"temp")==0) 
      OpenTempDialog(menu_item,NULL,NULL);
    else if (strcmp(name,"color")==0) 
      OpenColorDialog(menu_item,NULL,NULL);
    else if (strcmp(name,"zoom")==0)
      OpenZoomDialog(menu_item,NULL,NULL);
    else if (strcmp(name,"track")==0)
      OpenTrackDialog(menu_item,NULL,NULL);
    
    /*fprintf(stderr,"...opening complete\n");*/
    /*XtSetSensitive(menu_item, False);
    XtManageChild(wManager);
    XtPopup(XtParent(wManager),XtGrabNone);
    XSync(dpy, False);*/
  }
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* Auxiliary function for openparamdialog--forms a stringtable that
   contains all .events files in boundary's directory */
XmStringTable BuildTschemeList(int *n)
{
  XmStringTable strtable    ;
  FILE *cd;
  char list[MAXLINE][MAXLINE]/*={"item1","item2","item3","item4"}*/;
  char line[MAXLINE],temp[MAXLINE];
  int i;
  *n=0;
  system("ls -1 *.events 2>/dev/null 1>tmpfllst.txt");
  cd=fopen("tmpfllst.txt","r");
  while (fgets(line,MAXLINE,cd)!=NULL)  {
    sscanf(line,"%s",list[*n]);
    /*strcpy(list[*n],temp);*/
    (*n)++;
  }
  
  fclose(cd);
  system("rm tmpfllst.txt 2>/dev/null");
  strtable=(XmStringTable) XtMalloc((*n)* sizeof (XmString));
  for (i=0; i<*n;i++) 
    
    strtable[i] = XmStringCreateLocalized (list[i]);
  return strtable;
}


/*
 * Opens the parameters dialog.
 */
void OpenParamDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm, wForm2,wFrame, wLabel;
  Widget wRowCol, wRowCol2, wPushB;
  /*int scrn;*helios*/           
  Widget wToggle,wWin;
  /*tschemelist*/
  XmStringTable tschemeList;
  Widget wTschemeList;
  int tschemeN;
  
  wWin = CreateDialog(wTop,"Parameters",200,200,menu_item);

  /*main manager widgets*/
  wParamMan = CreatePane(wWin);
  wForm = CreateForm(wParamMan,5);
  wRowCol = CreateMainRowCol(wForm);

  /* simulation area */
  wFrame=CreateFrame(wRowCol);
  wForm2=CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Sim. area");
  wRowCol2 = CreateRCTA(wForm2, wLabel," x" ,SIZE_X);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2," y" ,SIZE_Y);

  /* boundary toggle button */
  wToggle = CreateToggleB(wRowCol,"Boundary",boundaryFlag,BOUNDARY);
  
  /*vertical boundary toggle button*/
  wToggle = CreateToggleB(wRowCol,"Vertical",verticalFlag,VERTIC);

  /*PeriodicX toggle button*/
  wPerxToggle = CreateToggleB(wRowCol,"PeriodicX",periodicXFlag,PERX);

  /* border width */
  /*wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel=CreateLabel(wForm2,"Fixed border width");
  wRowCol2 = CreateRCTA(wForm2, wLabel,"" ,BORDER);
*/
  /*Seaming PushB*/
  wSeamPushB = CreatePushB(wRowCol,"Seaming",SEAM);

  /* middle column begins*/
  wRowCol = CreateRightRowCol(wForm,wRowCol);
  
  /* lattice parameters */
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Lattice param.");
  wRowCol2 = CreateRCTA(wForm2, wLabel,"l1" ,L_PARAM);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"l2",L_PARAM2);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"Rel%" , RELAXATION);

  /* Potential */
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Potential");
  wRowCol2 = CreateRCTA(wForm2, wLabel,"e1" , EPSILON);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"e2" , EPSILON2);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"rc" , RCUT);
  
  /* Relaxation% *//*
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Relaxation (%)");
  wRowCol2 = CreateRCTA(wForm2, wLabel,"" , RELAXATION);*/

  /* right column begins*/
  wRowCol = CreateRightRowCol(wForm,wRowCol);

  /* Initial crack */
 
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Ev. file selection");
  wRowCol2 = CreateVRowCol(wForm2,wLabel);
  wToggle = CreateToggleB(wRowCol2,"Apply ev. file",tapplyFlag,TAPPLY);
  /*wRowCol2 = CreateRCTA(wForm2, wRowCol2,"x", CRACK_X);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"y", CRACK_Y);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"Width", CRACK_SX);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"Length", CRACK_SY);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"Angle", CRACK_ROT);
  */

  /*tscheme selection list*/
  tschemeList=BuildTschemeList(&tschemeN);
  
  wTschemeList = XmCreateScrolledList(wRowCol2, "_list",NULL,0);
				     /* tschemeList,tschemeN);*/
  XtVaSetValues(wTschemeList,   
     XmNitems, tschemeList,
     XmNitemCount, tschemeN,
     XmNvisibleItemCount, 4,
     /*XmNselectionPolicy,XmSINGLE_SELECT,  
     */NULL);
  
  XtManageChild(wTschemeList);
  XmListSelectPos(wTschemeList,1,True);
  XtAddCallback(wTschemeList,XmNdefaultActionCallback,CbList,NULL);
  XtAddCallback(wTschemeList,XmNbrowseSelectionCallback,CbList,NULL);

  wToggle = CreateToggleB(wRowCol2,"Adjust positions",tadjustFlag,TADJUST);
  wToggle = CreateToggleB(wRowCol2,"Adjust sizes",tadjust2Flag,TADJUST2);
  wToggle = CreateToggleB(wRowCol2,"Save events",savetscheme,TSAVE);
  /* OK and cancel PushButtons */
  wRowCol2 = CreateHRowCol(wRowCol,wRowCol);
  wPushBOK = CreatePushB(wRowCol2,"  OK  ",PARAM_OK);
  wPushBCancel = CreatePushB(wRowCol2,"Cancel",CANCEL);

  if (r == NULL)
    XtSetSensitive(wPushBCancel, False);

  /*TASensitive(CRACK_X, crackFlag);
  TASensitive(CRACK_Y, crackFlag);
  TASensitive(CRACK_SX, crackFlag);
  TASensitive(CRACK_SY, crackFlag);
  TASensitive(CRACK_ROT, crackFlag);*/
  
  /*scrn = DefaultScreen(dpy);*helios*/
/*  XtManageChild(wParamMan);*/
  
  /* realize the window */
  /*XtPopup (wWinParam, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wParamMan, menu_item, "param");

}


/*---------------------------------------------------------------*/
/*---------------------------------------------------------------*/

/* Opens the "fit the boundary" -dialog.*/

void OpenSeamDialog(Widget button,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm, wForm2, wFrame, wLabel,wToggle;
  Widget  wRowCol, wRowCol2, wPushB,wWin;
  /*int scrn;*helios*/     
  
  wWin = CreateDialog(wTop,"Seaming",600,200,button);
  
  /*main manager widgets*/
  wSeamMan = CreatePane(wWin);
  wForm = CreateForm(wSeamMan,5);
  wRowCol = CreateMainRowCol(wForm);
  
  /*Fixing suggestion toggle button*/
  wToggle = CreateToggleB(wRowCol,"Fixing suggestion",suggestFlag,SUGGEST);
  
  /* Boundary displacement */
  wFrame=CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Boundary displacement");
  wRowCol2 = CreateRCTA(wForm2, wLabel, "", BND_D);

  /*adjust upper/right lattice*/
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Adjust upper/right lattice");
  wRowCol2 = CreateRCTA(wForm2, wLabel,   "Move", BMOVE);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Rotate", ROT_R);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Cut up/right", UPCUT);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Cut down/left", DOWNCUT);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Narrow", DOT);

  /* right column of the dialog*/
  wRowCol = CreateRightRowCol(wForm,wRowCol);

  /* lattice displacement */
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Lattice displacement");
  wRowCol2 = CreateRCTA(wForm2, wLabel,"x", DISP_X);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2,"y", DISP_Y);

  /*bottom/left lattice adjustment*/
  wFrame=CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Adjust bottom/left lattice");
  wRowCol2 = CreateRCTA(wForm2, wLabel,   "Move", AMOVE);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Rotate", ROT_L);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Cut up/right", UPCUTA);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "Cut down/left", DOWNCUTA);
  

  /*OK button*/
  wRowCol2 = CreateHRowCol(wRowCol,wRowCol);
  wLabel = CreateLabel(wRowCol2,"          ");
  wPushB = CreatePushB(wRowCol2,"  OK  ",SEAM_OK);

  /*if (r == NULL) scrn = DefaultScreen(dpy);*helios*/   
  /* realize the window */
  /*XtAddCallback(wWinSeam,XmNpopdownCallback,CbCloseDialog,button);*/
  /*XtManageChild(wPane);
  XtPopup (wWinSeam, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wSeamMan, button, "seam");
}

/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/

/* Opens the animation dialog*/

void OpenAniDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm, wFrame, wLabel;
  Widget wRowCol2, wRowCol, wAPushB,wLimToggle;
  Widget wAniToggle;
  Widget wWin;

  wWin = CreateDialog(wTop,"Save animation",200,200,menu_item);
  
  /* main manager widgets*/
  wAniMan = CreatePane(wWin);
  wRowCol=CreateMainRowCol(wAniMan);
  
  /* Animation */
  wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm,"Animation");
  wRowCol2 = CreateVRowCol(wForm,wLabel);
  wAniToggle = CreateToggleB(wRowCol2,"Set animation",ani.aniFlag,SET_ANI);
  wRowCol2 = CreateRCTA(wForm, wRowCol2, "Wait    ", WAIT);

  wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm,"Set limits");
  wRowCol2 = CreateVRowCol(wForm,wLabel);

  /* Snapshot limit toggle button */
  wLimToggle = CreateToggleB(wRowCol2,"Limits on",ani.limitsFlag,LIM_TOGGLE);

  /* text display */
  wTextInfo2 = CreateTextInfo(wRowCol2,20,2,10,2);
  
  wRowCol2 = CreateRCTA(wForm, wRowCol2, "Begin at", BEGINAT);
  wRowCol2 = CreateRCTA(wForm, wRowCol2, "  End at", ENDAT);
  
  wRowCol2 = CreateVRowCol(wForm,wRowCol2);

  /* Snapshot end toggle button */
  wEndendToggle = CreateToggleB(wRowCol2,"Quit at end of ani",
				ani.endendFlag,ENDEND);
  
  wAPushB = CreatePushB(wRowCol,"   Close",CLOSE_ANI);
  /*if (r == NULL) scrn = DefaultScreen(dpy);*helios*/     

  /* realize the window */

  TASensitive(BEGINAT, ani.limitsFlag);
  TASensitive(ENDAT, ani.limitsFlag);
  XtSetSensitive(wEndendToggle, ani.limitsFlag);

  /*XtSetSensitive(menu_item,False);*/
  /*XtAddCallback(wWinAni,XmNpopdownCallback,CbCloseDialog,menu_item);*/
 /* XtManageChild(wPane);
  XtPopup(wWinAni, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wAniMan, menu_item, "ani");
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* Opens the strain etc/strain and pressure dialog*/

void OpenStrainDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm, wFrame, wLabel;
  Widget wRowCol2, wRowCol,wPushB;
  Widget wWin;

  wWin = CreateDialog(wTop,"Pressure",(int)(wDrawSize.x+150),0,
			    menu_item);
  
  /* main manager widgets for the whole dialog window*/
  wStrainMan = CreatePane(wWin);
  wRowCol = CreateMainRowCol(wStrainMan);
  
  /* strain */
  /*wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Strain");
  wRowCol2 = CreateVRowCol(wForm2,wLabel);
  wFixedToggle = CreateToggleB(wRowCol2,"Fix edges",fixedFlag,FIXED_TOGGLE);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "%", STRAIN_P);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "t", STRAIN_T);
  wRowCol2 = CreateVRowCol(wForm2,wRowCol2);
  wStrainPushB = CreatePushB(wRowCol2,"Apply",STRAIN);
  XtSetSensitive(wStrainPushB,fixedFlag);
*/
  /* pressure */
  wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm,"Pressure");
  wRowCol2 = CreateRCTA(wForm, wLabel, " ", PRESSURE);

  /* close dialog button*/
  wPushB = CreatePushB(wStrainMan,"CLOSE",CLOSE_STRAIN);
  
  /*XtSetSensitive(menu_item,False);
  XtAddCallback(wWinStrain,XmNpopdownCallback,CbCloseDialog,menu_item);
  XtManageChild(wPane);
  XtPopup(wWinStrain, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wStrainMan, menu_item, "strain");
}

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/

/* Opens the temperature (temperature controls and cooling) dialog.*/

void OpenTempDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm2, wFrame, wLabel;
  Widget wRowCol2, wRowCol, wPushB;
  Widget wWin;
  Widget wRandomizePushB;
  
  
  wWin = CreateDialog(wTop,"Temperature",(int)(wDrawSize.x+150),265,
			  menu_item);
  /* main manager widgets for the whole dialog window*/
  wTempMan = CreatePane(wWin);
  wRowCol = CreateMainRowCol(wTempMan);

  /* temperature */
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLowTempToggle = CreateToggleB(wForm2,"Force temperature to",
				 cool.TScaleFlag,TSCALE_TOGGLE);
  wRowCol2 = CreateRCTA(wForm2, wLowTempToggle, "K:", TEM);
  wRowCol2 = CreateVRowCol(wForm2,wRowCol2);
  wQuenchToggle = CreateToggleB(wRowCol2,"Quench to zero",cool.QuenchFlag,
				QUENCH_TOGGLE);
  wRandomizePushB = CreatePushB(wRowCol2,"Randomize",RANDOMIZE);
  
  /*Cooling Frame*/
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);
  wLabel = CreateLabel(wForm2,"Cooling parameters\n");
  wRowCol2 = CreateRCTA(wForm2, wLabel,   "Init temp", COOLTEMP);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, " Start at", STARTTIME);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "    Slope", SLOPE);
  wRowCol2 = CreateVRowCol(wForm2,wRowCol2);
  wCoolPushB = CreatePushB(wRowCol2,"  Start cooling  ",START_COOL);

  /*closing frame*/
  wForm2 = CreateForm(wRowCol,0);
  wPushB = CreatePushB(wForm2,"  CLOSE  ",CLOSE_TEMP);
  XtSetSensitive(menu_item,False);
  /*
  XtManageChild(wPane);
  XtPopup(wWinTemp, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wTempMan, menu_item, "temp");
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* Creates and opens the modify area dialog */
void OpenModDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm, wFrame, wLabel;
  Widget wRowCol2, wRowCol,wPushB;
  Widget /*wPane,*/wModifyMenu,wToggle,wWin;

  wWin = CreateDialog(wTop,"Modify area",(int)(wDrawSize.x+150),100,
			   menu_item);
  
  /* main manager widgets for the whole dialog window*/
  /*wPane*/wModMan = CreatePane(wWin);
  wRowCol = CreateMainRowCol(/*wPane*/wModMan);

  wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,0);
  wModifyMenu=CreateMenu(wForm,XmMENU_OPTION,"",'M',modifymenu);
  XtManageChild(wModifyMenu);

  wRowCol2 = CreateRCTA(wForm, wModifyMenu,"X", MODVX);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,   "Y", MODVY);
  TASensitive(MODVX, False);
  TASensitive(MODVY, False);

  wToggle = CreateToggleB(wRowCol,"Show selection",showselFlag,
				MODSHOWSEL);

  wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,1);
  wLabel = CreateLabel(wForm,"Area params");
  
  wRowCol2 = CreateRCTA(wForm, wLabel,  "x", MODAREA_X);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"y", MODAREA_Y);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"Width", MODAREA_SX);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"Length", MODAREA_SY);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"Angle", MODAREA_ROT);

  /*buttons*/
  wRowCol2 = CreateHRowCol(/*wPane*/wModMan,wRowCol2);
  wPushB = CreatePushB(wRowCol2,"   Modify   ",MODIFY);
  wPushB = CreatePushB(wRowCol2,"   CLOSE   ",CLOSE_MOD);
  modinfo=M_MODRM;
/*
  XtSetSensitive(menu_item,False);
  
  XtManageChild(wModMan);
  XtPopup(XtParent(wModMan), XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wModMan, menu_item, "mod");
}

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/


/* Opens the zoom dialog.*/

void OpenZoomDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm2, wFrame, wLabel;
  Widget wRowCol2, wRowCol, wPushB;
  Widget wWin,wPane;
 
  wWin = CreateDialog(wTop,"Zoom",(int)(wDrawSize.x+150),265,menu_item);
  
  /* main manager widgets*/
  wPane = CreatePane(wWin);
  wRowCol = CreateMainRowCol(wPane);
  wFrame = CreateFrame(wRowCol);
  wForm2 = CreateForm(wFrame,0);

  /*view size*/
  wLabel = CreateLabel(wForm2,"View size");
  wRowCol2 = CreateRCTA(wForm2, wLabel,   "x", ZOOM_X);
  wRowCol2 = CreateRCTA(wForm2, wRowCol2, "y", ZOOM_Y);

  /*close*/
  wForm2 = CreateForm(wRowCol,0);
  wPushB = CreatePushB(wForm2,"CLOSE",CLOSE_ZOOM);
  XtSetSensitive(menu_item,False);
  
  XtManageChild(wPane);
  XtPopup(wWin, XtGrabNone);
  XSync(dpy, False);
}

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/

/* Opens the color dialog */

void OpenColorDialog(Widget menu_item,XtPointer client_data,
		     XtPointer call_data)
{
  Widget wForm, wLabel;
  Widget wRowCol2, wRowCol,wPushB;
  Widget wWin,wToggle,wToggle2;

  wWin = CreateDialog(wTop,"Color ranges",(int)(wDrawSize.x+150),100,
			   menu_item);
  
  /* main manager widgets*/
  wColorMan = CreatePane(wWin);
  wRowCol = CreateMainRowCol(wColorMan);

  /*toggles*/
  wToggle = CreateToggleB(wRowCol,"Auto-calc colors",
				 colorranges.Update,COLRANUPD);
  wToggle2 = CreateToggleB(wRowCol,"Hide atoms outside",
				 colorranges.hide,COLRANHIDE);

  /*color ranges*/
  wLabel = CreateLabel(wRowCol,"color ranges:");
  wForm = CreateForm(wRowCol,2);
  wRowCol2 = CreateRCTA(wForm, wLabel,  "EkMin", COLRANKMIN);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"EkMax", COLRANKMAX);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"EpMin", COLRANPMIN);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"EpMax", COLRANPMAX);
  wRowCol2 = CreateRCTA(wForm, wRowCol2," SMin", COLRANSMIN);
  wRowCol2 = CreateRCTA(wForm, wRowCol2," SMax", COLRANSMAX);
  wRowCol2 = CreateRCTA(wForm, wRowCol2," FMin", COLRANFMIN);
  wRowCol2 = CreateRCTA(wForm, wRowCol2," FMax", COLRANFMAX);
  TASensitive(COLRANKMIN, !colorranges.Update);
  TASensitive(COLRANKMAX, !colorranges.Update);
  TASensitive(COLRANPMIN, !colorranges.Update);
  TASensitive(COLRANPMAX, !colorranges.Update);
  TASensitive(COLRANSMIN, !colorranges.Update);
  TASensitive(COLRANSMAX, !colorranges.Update);
  TASensitive(COLRANFMIN, !colorranges.Update);
  TASensitive(COLRANFMAX, !colorranges.Update);

   /* close dialog button*/
  wPushB = CreatePushB(wColorMan,"CLOSE",CLOSE_COLOR);
  
  /*XtSetSensitive(menu_item,False);*/
  /*XtAddCallback(wWinCrack,XmNpopdownCallback,CbCloseDialog,menu_item);*/
  /*XtManageChild(wPane);
  XtPopup(wWinColor, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wColorMan, menu_item, "color");
}

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/

/* Opens the tracking dialog */
void OpenTrackDialog(Widget menu_item,XtPointer client_data,
			   XtPointer call_data)
{
  Widget wForm, wFrame, wLabel, wTextSave;
  Widget wRowCol2, wRowCol, wPushB,wToggle,wWin;
  
  wWin = CreateDialog(wTop,"Track defects",100,300,menu_item);
  
  /* main manager widgets for the whole dialog window*/
  wTrackMan = CreatePane(wWin);
  wRowCol = CreateMainRowCol(wTrackMan);
  wFrame = CreateFrame(wRowCol);
  wForm = CreateForm(wFrame,0);

  wLabel = CreateLabel(wForm,"Tracking parameters");
  wRowCol2 = CreateVRowCol(wForm,wLabel);

  /* Setup tracking toggle button */
  wToggle = CreateToggleB(wRowCol2,"Track it!",SetTrackFlag,SETTRACKTOG);

  /* text display */
  wTextInfoT = CreateTextInfo(wRowCol2,20,2,10,2);

  wRowCol2 = CreateRCTA(wForm, wRowCol2,"Range", TRANGE);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"SFactor", TFACTOR);
  wRowCol2 = CreateRCTA(wForm, wRowCol2,"BorderF", TBFACTOR);
  
  wRowCol2 = CreateVRowCol(wForm,wRowCol2);
  
  /* Save toggle button */
  wSaveTrackToggle = CreateToggleB(wRowCol2,"Save positions",
				   saveTrackFlag,SAVETRK);
 /* Show toggle button */
  wShowTrackToggle = CreateToggleB(wRowCol2,"Show positions",
				   saveTrackFlag,SHOWTRK);

  wTextSave =  XtVaCreateManagedWidget("wTextSave",
    xmTextFieldWidgetClass, wRowCol2,
    XmNtopAttachment,       XmATTACH_WIDGET,
    XmNtopWidget,           wShowTrackToggle,  
    NULL);
  
  XtAddCallback(wTextSave, XmNactivateCallback,CbSaveTra ,NULL);

  wPushB = CreatePushB(wRowCol,"  Close  ",CLOSE_TRK);
  /*if (r == NULL) scrn = DefaultScreen(dpy);*helios*/  

  /* realize the window */
  
  TASensitive(TRANGE, SetTrackFlag);
  TASensitive(TFACTOR, SetTrackFlag);
  TASensitive(TBFACTOR,SetTrackFlag); 
  XtSetSensitive(wSaveTrackToggle, SetTrackFlag);
  XtSetSensitive(wShowTrackToggle, SetTrackFlag);

  /*XtSetSensitive(menu_item,False);*/
  /*XtAddCallback(wWinTrack,XmNpopdownCallback,CbCloseDialog,menu_item);*/
  /*XtManageChild(wPane);
  XtPopup(wWinTrack, XtGrabNone);
  XSync(dpy, False);*/
  OpenDialog(wTrackMan, menu_item, "track");
}

/*---------------------------------------------------------------*/
/*---------------------------------------------------------------*/


/*initializes a named color*/
#define INITCOLOR(index,name) {XParseColor(dpy,cmap,name,&Color); \
  XAllocColor(dpy,cmap,&Color); \
  color[index]=Color.pixel;}

/*--------------------------------------------------------------*/
/*--------------------------------------------------------------*/


/*
 * setup colors
 */
void SetupColors(int bpp)
{
  int i,rb,gb,bb;
  double h,s,l,r,g,b;
  static double hmin=0.67,hmax=0.0;
  static double lmin=0.4,lmax=0.4;

  Colormap cmap;
  XColor Color;
  int scrn;
  char cname[13];
  dpy=XtDisplay(wTop);
  scrn = DefaultScreen(dpy);
  cmap=DefaultColormap(dpy,scrn);

  INITCOLOR(COLORS,"black");
  INITCOLOR(BLACK,"black");
  INITCOLOR(BACKGROUND,"white");
  INITCOLOR(RED,"red");
  INITCOLOR(BLUE,"blue");
  INITCOLOR(FIXEDB,"green");
  INITCOLOR(FIXEDT,"orange");
  INITCOLOR(FORCE1,"skyblue");
  INITCOLOR(FORCE2,"red4");
  INITCOLOR(SELECT,"gray");

  if (bpp == 16){
    
    s=1.0;
    for (i = 0; i < COLORS; i++) {
      h=hmin+(double)i/(double)COLORS*(hmax-hmin);
      l=lmin+(double)i/(double)COLORS*(lmax-lmin);
      HSL_to_RGB(h,s,l,&r,&g,&b);
      rb=(int)(r*(double)MAXRGB);
      gb=(int)(g*(double)MAXRGB);
      bb=(int)(b*(double)MAXRGB);
      sprintf(cname,"rgb:%04x/%04x/%04x",rb,gb,bb);
      if (!XParseColor(dpy,cmap,cname,&Color)) {
	fprintf(stderr,"Error in color parsing: %d %s\n",i,cname);
	bexit(1);
      }
      if (!XAllocColor(dpy,cmap,&Color)) {
	fprintf(stderr,"Error in color allocation: %d   %d %d %d\n",
		i,Color.red,Color.green,Color.blue);
	bexit(1);
      }
      RGB[i].r=r;
      RGB[i].g=g;
      RGB[i].b=b;
      color[i]=Color.pixel;
    }
  }
  
  else {
    for (i = 0; i < COLORS; i++)
      INITCOLOR(i,"blue");
  }
}


/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/

/*Drawranges is macro that draws the plot range texts in the middle area*/ 

#define DRAWRANGES(x1,y1,x2,y2,c1,c2) { \
   XSetForeground(dpy, gc, color[c1]);\
   XDrawString(dpy,rangepixmap,gc,x1,y1,"Epot ",\
     strlen("Epot "));\
   XDrawString(dpy,rangepixmap,gc,x1+30,y1,bufp,strlen(bufp));\
   XSetForeground(dpy, gc, color[c2]);\
   XDrawString(dpy,rangepixmap,gc,x2,y2,"Ekin ",\
     strlen("Ekin "));\
   XDrawString(dpy,rangepixmap,gc,x2+30,y2,bufk,strlen(bufk));}

/* Drawpoint draws a line to the screen. Used in plot drawing.*/ 

#define DRAWPOINT(x1,y1,x2,y2,c) {XSetForeground(dpy, gc, color[c]);\
                                  XDrawLine(dpy,plpixmap,gc,x1,y1,x2,y2);}


/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/

/* Round a double to nearest integral double, then converts it to integer
   and returns. Used in drawing to ensure that the lines are drawed in correct
   positions, because pixel coordinates are integers, not doubles.*/
int gtype(double d) 
{
  return ((int) nint(d));
}

/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/

/*Draws the colorbar in the middle area which shows atom color coding.*/

void DrawColorBar() {
  unsigned int xcbar,ycbar;
  int i,j;
  double dcol;
  
  xcbar=(unsigned int)(wRangeSize.x/2-COLORBARWIDTH/2);
  ycbar=0;
  /* Draw the colorbar */
  for (i=0;i<COLORS;i++) {
    dcol=(double)i/(double)COLORS;
    if ((sinfo != STYPE) && (sinfo != SVECT)) 
      j=(int)((double)(COLORS-1)*calccolindex(dcol,0.0,1.0));
    else {
      if (i>COLORS/2)
	j = RED;
      else
	j = BLUE;
      /*j=(i<COLORS/2)?0:BLUE;*/
    }
	
    XSetForeground(dpy, gc, color[j]);
     
    XFillRectangle(dpy,rangepixmap,gc,xcbar+gtype(COLORBARWIDTH*dcol),ycbar,
		   (unsigned int) gtype(1+COLORBARWIDTH/COLORS),
		   gtype(wRangeSize.y));
  }
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*Draws the plot in the bottom area. Arg init is 1 when the plot is drawn
  for the first time. After that it is 0.*/

void RedrawPlot(int init)
{
  int i,kcopy,pcopy,updateranges;
  static double Epplotmin=-6.55,Epplotmax=-6.25;
  static double Ekplotmin=0.0,Ekplotmax=0.08;
#define KNMAX 2000  /*maximum plot history size*/ 
  static VecD kien[KNMAX],poen[KNMAX];
  double kienmin,kienmax,poenmin,poenmax;
  static char bufp[50],bufk[50];
  static int xstr1=10,xstr2=350,ystr1=10,ystr2=10;
  static double prevplotxscale;
  double ustep,x,y,xprev,yprev;
  static int cindex;
  VecD pSize;
  
  updateranges=0;  /*updateranges is 1 when the plot scaling changes*/   
  if (init>0) {
    updateranges=1;
    xstr2 = gtype(wDrawSize.x/2+COLORBARWIDTH/2+10);
  }
  kcopy=1;
  pcopy=1;
  if (updateStep/plotdata.xscale<=1.0) {
    kcopy=0;
    pcopy=0;
  }
  if (plotdata.xscale != prevplotxscale) {
    pcopy=0;
    kcopy=0;
  }
  ustep=(double) updateStep;
  pSize.x=(double) wPlotSize.x;
  pSize.y=(double) wPlotSize.y;
  prevplotxscale=plotdata.xscale;
  
  /*minimizing-scaling*/
  /*Global variable plotdata.rescale is 1 when the user selects rescale 
    plot from the view menu. Then the plot vertical range is minimized 
    and the variable is set back to 0.*/
  if (plotdata.rescale && (init==0)) {
    poenmin=potEnergy;
    poenmax=potEnergy;
    kienmin=kinEnergy;
    kienmax=kinEnergy;
    if (cindex==0)
      i=KNMAX-1;
    else
      i=cindex-1;
    xprev= pSize.x-1;
    
    while((i != cindex) && (xprev>=pSize.x/2)) {
      if (kien[i].y<kienmin) kienmin=kien[i].y;
      if (kien[i].y>kienmax) kienmax=kien[i].y;
      if (poen[i].y<poenmin) poenmin=poen[i].y;
      if (poen[i].y>poenmax) poenmax=poen[i].y;
      if (i==KNMAX-1)
	x = xprev-(kien[0].x)/plotdata.xscale;
      else
	x = xprev-(kien[i+1].x)/plotdata.xscale;
      xprev=x;
      i--;
      if (i<0) i=KNMAX-1;
    }

    Epplotmin=poenmin;
    Epplotmax=poenmax;
    Ekplotmax=kienmax;
    Ekplotmin=kienmin;
    pcopy=0;
    kcopy=0;
    plotdata.rescale=0;
    updateranges=1;
  }
  
  /*Scaling. If a new plot value is over the maximum or below the minimum, the
   plot is automatically scaled so that all values fit. The plot does NOT 
   automatically scale back to minimum possible range that shows all values.
   User must command this from the menu.*/
  
  if (kinEnergy>Ekplotmax) {
    Ekplotmax=plotdata.yscalingratio*(kinEnergy-Ekplotmin)+Ekplotmin;
    kcopy=0;
    updateranges=1;
  }
  if (kinEnergy<Ekplotmin) {
    Ekplotmin=Ekplotmax-plotdata.yscalingratio*(Ekplotmax-kinEnergy);
    kcopy=0;
    updateranges=1;
    
  }
  if (Ekplotmin<0.01) Ekplotmin=0;
  if (potEnergy>Epplotmax) {
    Epplotmax=plotdata.yscalingratio*(potEnergy-Epplotmin)+Epplotmin;
    pcopy=0;
    updateranges=1;
  }
  if (potEnergy<Epplotmin) {
    Epplotmin=Epplotmax-plotdata.yscalingratio*(Epplotmax-potEnergy);
    pcopy=0;
    updateranges=1;
  }
  if (Epplotmax-Epplotmin<MINPLOTRANGE) Epplotmin=Epplotmax-MINPLOTRANGE;
  if (Ekplotmax-Ekplotmin<MINPLOTRANGE) Ekplotmax=Ekplotmin+MINPLOTRANGE;
  
  if (updateranges) {
    /*prev all %8.4g...*/
    sprintf(bufp,"%4.4g - %4.4g",Epplotmin,Epplotmax);
    sprintf(bufk,"%4.4g - %4.4g",Ekplotmin,Ekplotmax);
    XSetForeground(dpy, gc, color[BACKGROUND]);
    XFillRectangle(dpy, rangepixmap, gc, 0, 0, wRangeSize.x, wRangeSize.y); 
    DRAWRANGES(xstr1,ystr1,xstr2,ystr2,RED,BLUE);
    DrawColorBar();
  }
  
  /* Handle the plotting area */
  if (init>0) {
    kcopy=0;
    pcopy=0;
    plotdata.rescale=1;
    
    for(i=0;i<=(KNMAX-1);i++) {
      kien[i].x=10;
      kien[i].y=kinEnergy;
      poen[i].x=10;
      poen[i].y=potEnergy;
    }
  }
      
  cindex++;
  if (cindex>KNMAX-1) cindex=0;
  
  kien[cindex].y=kinEnergy;
  kien[cindex].x=ustep;
    
  poen[cindex].y=potEnergy;
  poen[cindex].x=ustep;
  
  /*kinetic energy plotting*/

  /*calc coordinates for the new point*/
  xprev= pSize.x-1;
  yprev= pSize.y-(kien[cindex].y-Ekplotmin)/(Ekplotmax-Ekplotmin)
    *pSize.y;
  if (!kcopy) { /*choose this branch if the plot has to drawn completely*/
    XSetForeground(dpy, gc, color[BACKGROUND]);
    XFillRectangle(dpy, plpixmap, gc, gtype(pSize.x/2), 0, gtype(pSize.x/2), 
		   wPlotSize.y);
    if (cindex==0)
      i=KNMAX-1;
    else
      i=cindex-1;
    
    while((i != cindex) && (xprev>=pSize.x/2)) {
      y = pSize.y-(kien[i].y-Ekplotmin)/(Ekplotmax-Ekplotmin)*pSize.y; 
      if (i==KNMAX-1)
	x = xprev-(kien[0].x)/plotdata.xscale;
      else
	x = xprev-(kien[i+1].x)/plotdata.xscale;
      if (x >= pSize.x/2) {
	if (yprev-1<0) yprev=yprev+1;
	if (y-1<0) y=y+1;
	DRAWPOINT(gtype(xprev),gtype(yprev-1),gtype(x),gtype(y-1),BLUE);
      }
      xprev=x;
      yprev=y;
      i--;
      if (i<0) i=KNMAX-1;
    }
  }
  else {  /*choose this branch if one value has to be added to plot and the
	    scaling has not changed since last time. This is MUCH faster
	    than the first branch.*/
    x=pSize.x- ustep/plotdata.xscale;
    if (cindex>0) {
      y=pSize.y-(kien[cindex-1].y-Ekplotmin)/(Ekplotmax-Ekplotmin)
	*pSize.y;
    }
    else {
      y=pSize.y-(kien[KNMAX-1].y-Ekplotmin)/(Ekplotmax-Ekplotmin)
	*pSize.y;
    }
    XCopyArea(dpy, plpixmap, plpixmap, gc, 
	      gtype(pSize.x/2+ustep/plotdata.xscale), 0, 
	      gtype(1+pSize.x/2- ustep/plotdata.xscale), wPlotSize.y, 
	      gtype(pSize.x/2+1), 0);
    XSetForeground(dpy, gc, color[BACKGROUND]);
    XFillRectangle(dpy, plpixmap, gc,gtype(x+0.9), 0, 
		   gtype(1+ustep/plotdata.xscale), wPlotSize.y);
    if (yprev-1<0) yprev=yprev+1;
    if (y-1<0) y=y+1;
    DRAWPOINT(gtype(xprev),gtype(yprev-1),gtype(x),gtype(y-1),BLUE);
  }
  
  /*potential energy plotting*/

  /*calc coordinates for the new point*/
  xprev= pSize.x/2-1;
  yprev= pSize.y-(poen[cindex].y-Epplotmin)/(Epplotmax-Epplotmin)
    *pSize.y;
  if (!pcopy) {   /*choose this branch if the plot has to drawn completely*/
    XSetForeground(dpy, gc, color[BACKGROUND]);
    XFillRectangle(dpy, plpixmap, gc, 0, 0, gtype(pSize.x/2), 
		     wPlotSize.y);
    if (cindex==0)
      i=KNMAX-1;
    else
      i=cindex-1;
    
    while((i != cindex) && (xprev>=0)) {
      y = pSize.y-(poen[i].y-Epplotmin)/(Epplotmax-Epplotmin)*pSize.y; 
      if (i==KNMAX-1)
	x = xprev-(poen[0].x)/plotdata.xscale;
      else
	x = xprev-(poen[i+1].x)/plotdata.xscale;
      if (x >= 0) {
	if (yprev-1<0) yprev=yprev+1;
	if (y-1<0) y=y+1;
	DRAWPOINT(gtype(xprev),gtype(yprev-1),gtype(x),gtype(y-1),RED);
      }
      xprev=x;
      yprev=y;
      i--;
      if (i<0) i=KNMAX-1;
      
    }
  }
  else {  /*choose this branch if one value has to be added to plot and the
	    scaling has not changed since last time. This is MUCH faster
	    than the first branch.*/
    x=pSize.x/2- ustep/plotdata.xscale;
    if (cindex>0)
      y=pSize.y-(poen[cindex-1].y-Epplotmin)/(Epplotmax-Epplotmin)
	*pSize.y; 
    else
      y=pSize.y-(poen[KNMAX-1].y-Epplotmin)/(Epplotmax-Epplotmin)
	*pSize.y;
    XCopyArea(dpy, plpixmap, plpixmap, gc, 
	      gtype(ustep/plotdata.xscale), 0, 
	      gtype(1+pSize.x/2- ustep/plotdata.xscale), wPlotSize.y, 
	      1, 0);
    XSetForeground(dpy, gc, color[BACKGROUND]);
    XFillRectangle(dpy, plpixmap, gc,gtype(x+0.9), 0, 
		   gtype(1+ustep/plotdata.xscale), wPlotSize.y);
    
    if (yprev-1<0) yprev=yprev+1;
    if (y-1<0) y=y+1;
    DRAWPOINT(gtype(xprev),gtype(yprev-1),gtype(x),gtype(y-1),RED);
  }
  
  RefreshPlot(); /*refresh the plot on screen*/
}
 
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/


/*
 * Draw atoms on the screen
 */

/*#define FILL_DISK(x, y, diam, pixmap) \
  XFillArc(dpy, (pixmap), gc, (x), (y), (diam).x, (diam).y, 0, 360 * 64)*/
#define DRAW_ARROW(x1, y1, x2, y2, pixmap) \
  XDrawLine(dpy, (pixmap), gc, (x1), (y1), (x2), (y2)), \
  XDrawLine(dpy, (pixmap), gc, (x2), (y2), (x2) - 2, (y2) - 1), \
  XDrawLine(dpy, (pixmap), gc, (x2), (y2), (x2) - 1, (y2) - 2), \
  XDrawArc(dpy, (pixmap), gc, (x1) - diamPix.x / 6 , (y1) - diamPix.y / 6, \
          (diamPix).x/3, (diamPix).y/3, 0, 360 * 64)   
     
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*Updates all textinfos that has to be dynamically updated.*/
void UpdateTextInfos(double colormin,double colormax)
{
  char buff[250], anim[50], trackinfo[50];
  double countit; /*amount of snapshots*/

  sprintf(buff, "Time:  %.0f\nAtoms: %d\nSize:\n %.1f x %.1f\nCenter:\n %.1f x %.1f\nEtot: %.6f\nEpot: %.6f\nEkin: %.6f\nTemp: %.5f\nColor range:\n %+g\n %+g",
	  (float)(stepCount * 1000 *deltaT), nAtom,xMax-xMin,yMax-yMin,
	  center.x,center.y,kinEnergy+potEnergy,potEnergy,kinEnergy,
	  UNITTEMP*kinEnergy,colormin,colormax);
  XmTextSetString(wTextInfo, buff);
  
  if(wWinAniFlag){
    if(ani.limitsFlag){
      countit=0.0+(int)(1.0+(ani.endat-ani.beginat)/ani.waittime);
    }
    sprintf(anim, "Snapshots: %.0f ",countit);
    XmTextSetString(wTextInfo2, anim);
  }
  
  /*-------Contain of the text info in track window----*/
  if(wWinTrackFlag){
    if(NumberofDefects>0){
      sprintf(trackinfo, "No. of defects: %.0ld ",(long int)NumberofDefects);
      XmTextSetString(wTextInfoT, trackinfo);}
    else {
      sprintf(trackinfo, "No. of defects: 0 ");
      XmTextSetString(wTextInfoT, trackinfo);}
  }
}
     
/*------------------------------------------------------------------------*/
/*------------------------------------------------------------------------*/

/*Redraws the main simulation atom view area. Draws atoms or vectors.*/
void Redraw()
{  
  int n, colIndex/*,changecol*helios*/;
  /*char buff[250];*helios*/
  char DefLabel[10]; 
  double avVel;
  int at,dind,llen;
  double ctmp,ctmpmin,ctmpmax;
#ifdef DRAWHEXAGONS
  XPoint points[6];           
  double s3;          
  int rpixx,rpixy;   
#endif
  int SaveAllTracks=1;  /*if >0 all defects are saved*/
  int ind;
  VecD scale, edgemin, edgemax;
  VecI rPix, diamPix, offsPix;
  VecD crackc,cracks,crackp1,crackp2,crackp3,crackp4;
  VecD v;
#ifdef SHOWINDEX
  char buff[10];
#endif
#ifdef DRAWHEXAGONS
  s3=sqrt(3.0);    
#endif
  
  /* Get color coding range and mapping info */
  if (sinfo == SKIN) {        /* Kinetic energy */
    cix0=cix0o;
    ciy0=ciy0o;
    if (colorranges.Update) {
      colorranges.EkMin=kinEnmin;
      colorranges.EkMax=kinEnmax;
    }
    ctmpmin=colorranges.EkMin;
    ctmpmax=colorranges.EkMax;
  } 
  else if (sinfo == SPOT) { /* Potential energy */
    cix0=cix0p;
    ciy0=ciy0p;
    if (colorranges.Update) {
      colorranges.EpMin=potEnmin;
      colorranges.EpMax=potEnmax;
    }
    ctmpmin=colorranges.EpMin;
    ctmpmax=colorranges.EpMax;
  } 
  else if (sinfo == SFORCE) { /* Total force/acceleration */
    cix0=cix0o;
    ciy0=ciy0o;
    if (colorranges.Update) {
      colorranges.FMin=forcemin;
      colorranges.FMax=forcemax;
    }
    ctmpmin=colorranges.FMin;
    ctmpmax=colorranges.FMax;
  } 
  else {            /* Stress component */
    cix0=cix0o;
    ciy0=ciy0o;
    if (colorranges.Update) {
      colorranges.SMin=stressijmin;
      colorranges.SMax=stressijmax;
    }
    ctmpmin=colorranges.SMin;
    ctmpmax=colorranges.SMax;
  }
  
  /* clear pixmap */
  
  XSetForeground(dpy, gc, color[BACKGROUND]);
  XFillRectangle(dpy, pixmap, gc, 0, 0, wDrawSize.x, wDrawSize.y);

  scale.x = (double)wDrawSize.x / zoom.x;
  scale.y = (double)wDrawSize.y / zoom.y;
  diamPix.x = (int)(diam * lParam * scale.x / sqrt(2.0));
  if (diamPix.x < 2) diamPix.x = 2;
  diamPix.y = (int)(diam * lParam * scale.y / sqrt(2.0));
  if (diamPix.y < 2) diamPix.y = 2;
  if (diamPix.x > diamPix.y)
    diamPix.x = diamPix.y;
  else
    diamPix.y = diamPix.x;
  
  offsPix.x = wDrawSize.x / 2 - (int)nint(scale.x * center.x);
  offsPix.y = wDrawSize.y / 2 + (int)nint(scale.y * center.y);
  edgemin.x = -0.5 * (zoom.x + diam * lParam / sqrt(2.0)) + center.x;
  edgemin.y = -0.5 * (zoom.y + diam * lParam / sqrt(2.0)) + center.y;
  edgemax.x = 0.5 * (zoom.x + diam * lParam / sqrt(2.0)) + center.x;
  edgemax.y = 0.5 * (zoom.y + diam * lParam / sqrt(2.0)) + center.y;
  avVel = sqrt(2.0 * kinEnergy);

  if (sinfo == SVECT) {   /*draw vectors instead of atoms*/

    /* Draw vectors */
    if (avrCut >= 1.0) {  /*average vectors*/
      Average(rv);           
      diamPix.x *= 2.0 * avrCut;
      diamPix.y *= 2.0 * avrCut;
      for (n = 0; n < avCells.x * avCells.y; n++) {
      	if (avatype[n]>0) {
	  XSetForeground(dpy, gc, color[RED]);
	} else {
	  XSetForeground(dpy, gc, color[BLUE]);
	}
	if (avr[n].x > edgemin.x && avr[n].x < edgemax.x && 
	    avr[n].y > edgemin.y && avr[n].y < edgemax.y) {
	  rPix.x = (int)nint(scale.x * avr[n].x) + offsPix.x;
	  rPix.y = -(int)nint(scale.y * avr[n].y) + offsPix.y;
	  DRAW_ARROW(rPix.x,rPix.y,
		     rPix.x+(int)nint((double)diamPix.x*avv[n].x/avVel),
		     rPix.y-(int)nint((double)diamPix.y*avv[n].y/avVel),
		     pixmap);
	}
      } /* for */
    }    /* if (avrCut... */
    else { /* vectors, !averaging */  
      for (n = 0; n < nAtom; n++) {
	if (atype[n]==1 || atype[n]==(-1)){
	  if (fixvel[n]==0) XSetForeground(dpy, gc, color[BLUE]);
	  else XSetForeground(dpy, gc, color[FIXEDB]);
	} else {
	  if (fixvel[n]==0) XSetForeground(dpy, gc, color[RED]);
	  else XSetForeground(dpy, gc, color[FIXEDT]);
	}
	if (r[n].x > edgemin.x && r[n].x < edgemax.x &&
	    r[n].y > edgemin.y && r[n].y < edgemax.y) {
	  rPix.x = (int)nint(scale.x * r[n].x) + offsPix.x;
	  rPix.y = -(int)nint(scale.y * r[n].y) + offsPix.y;
	  DRAW_ARROW(rPix.x,rPix.y,
		     rPix.x+(int)nint((double)diamPix.x*rv[n].x/avVel),
		     rPix.y-(int)nint((double)diamPix.y*rv[n].y/avVel),pixmap);
	}
      } /* for */
    } /* else */
  } 
  else { 
    /* Draw atoms, not vectors */
    offsPix.x -= diamPix.x / 2;
    offsPix.y -= diamPix.y / 2;

    for (n = 0; n < nAtom; n++) {
      at=ABS(atype[n]);
      if (r[n].x > edgemin.x && r[n].x < edgemax.x &&
	  r[n].y > edgemin.y && r[n].y < edgemax.y) { 
	rPix.x = (int)nint(scale.x * r[n].x) + offsPix.x;
	rPix.y = -(int)nint(scale.y * r[n].y) + offsPix.y;
	if (fixedFlag && at<0) colIndex = BLACK;

	/* Show kinetic energy */
	if ((sinfo == SKIN) && runningFlag ) {
	  VSAdd(v, rv[n], -0.5 * deltaT, ra[n]);
	  if (ctmpmax-ctmpmin>SMALLDOUBLE)
	    colIndex = gtype((0.5*VLenSq(v)-ctmpmin)
			    /(ctmpmax-ctmpmin) *(COLORS-1));
	  else 
	    colIndex = COLORS/2;
	  if (colIndex>=COLORS) {
	    if (colorranges.hide) colIndex=-1;
	    else colIndex=(COLORS-1);
	  }
	  else if (colIndex < 0) {
	    if (colorranges.hide) colIndex=-1;
	    else colIndex=0;
	  }
	/* show force/acceleration */
	} else if ((sinfo == SFORCE) && runningFlag ) {
	  if (ctmpmax-ctmpmin>SMALLDOUBLE)
	    colIndex = gtype((sqrt(VLenSq(ra[n]))-ctmpmin)
			    /(ctmpmax-ctmpmin) *(COLORS-1));
	  else 
	    colIndex = COLORS/2;
	  if (colIndex>=COLORS) {
	    if (colorranges.hide) colIndex=-1;
	    else colIndex=(COLORS-1);
	  }
	  else if (colIndex < 0) {
	    if (colorranges.hide) colIndex=-1;
	    else colIndex=0;
	  }
	/* Show potential energy or stress */
	} else if ((sinfo==SPOT || sinfo==SXX || sinfo==SXY || sinfo==SYY) 
		   && runningFlag) { 
	  if (sinfo == SPOT) ctmp=potEn[n]; else ctmp=stressij[n];
	  if (ctmpmin>=ctmpmax) {
	    colIndex =(int)((COLORS+(4*ctmp))/2.0-1.0);
	    if (colIndex >= COLORS/2) colIndex = (COLORS/2 - 1);
	    if (colIndex <0) colIndex =0;
	  } else {
	    colIndex=(int)((double)(COLORS-1)
			   *calccolindex(ctmp,ctmpmin,ctmpmax));
	  }
	  if (colIndex>=COLORS) {
	    if (colorranges.hide) colIndex=-1;
	    else colIndex=(COLORS-1);
	  }
	  else if (colIndex < 0) {
	    if (colorranges.hide) colIndex=-1;
	    else colIndex=0;
	  }

	  /* Show atom species */
	} else if (sinfo == STYPE || !runningFlag) {
	  if (at==1) {
	    if (fixvel[n]==0) colIndex = BLUE;
	    else colIndex = FIXEDB;
	    if (cforce[n].x != 0 || cforce[n].y != 0)
	      colIndex = FORCE1;
	  } else {
	    if (fixvel[n]==0) colIndex = RED;
	    else colIndex = FIXEDT;
	    if (cforce[n].x != 0 || cforce[n].y != 0)
	      colIndex = FORCE2;
	  }
	}
	if (wWinModFlag && showselFlag  && Selected(n,modarea)) 
	  colIndex = SELECT;
	
	if (colIndex>-1) {
	  XSetForeground(dpy, gc, color[colIndex]);
#ifdef DRAWHEXAGONS   	
	  rpixx=diamPix.x/4; if (rpixx==0) rpixx=1;
	  rpixy=diamPix.y/4; if (rpixy==0) rpixy=1;
	  points[0].x=  rPix.x+s3*rpixx;  points[0].y=  rPix.y+  rpixy; 
	  points[1].x=  rPix.x;           points[1].y=  rPix.y+2*rpixy; 
	  points[2].x=  rPix.x-s3*rpixx;  points[2].y=  rPix.y+  rpixy; 
	  points[3].x=  rPix.x-s3*rpixx;  points[3].y=  rPix.y-  rpixy; 
	  points[4].x=  rPix.x;           points[4].y=  rPix.y-2*rpixy; 
	  points[5].x=  rPix.x+s3*rpixx;  points[5].y=  rPix.y-  rpixy; 
	  XFillPolygon(dpy,pixmap,gc,points,6,Convex,CoordModeOrigin);
#else
	  XFillArc(dpy, pixmap, gc, rPix.x, rPix.y, diamPix.x, diamPix.y, 
		   0,360 * 64);
#endif
	
#ifdef SHOWINDEX
	llen=(int)(log((double)(n>0?n:1))/log(10.0))+1;
	sprintf(buff,"%d",n);
	XSetBackground(dpy,gc,color[BACKGROUND]);
	XSetForeground(dpy,gc,color[BLACK]);
	XDrawImageString(dpy,pixmap,gc,rPix.x+diamPix.x/2,rPix.y+diamPix.y/2,
			 buff,llen);
#endif	
	}
      }
    }
  }
  
/*---------Draw the CM of each defect-set------------------------------------*/
  if (SetTrackFlag && NumberofDefects>0) {
    for (ind=0;ind<NumberofDefects;ind++) {
      if (SaveAllTracks) {
	savetrack(ind, numero);
      } else {
	for( n = 0; n < 5; n++){	
	  if(numero[n]==label1[ind] && (long int) ( stepCount )% 8 ==0){
	    savetrack(ind, numero);
	  }
	}
      }
      if (showTrackFlag) {
	rPix.x = (int)nint(scale.x * rTrk[ind].x) + offsPix.x;
	rPix.y = -(int)nint(scale.y * rTrk[ind].y) + offsPix.y;
	XSetForeground(dpy, gc, color[BLACK]);
	XSetBackground(dpy, gc, color[RED-1]);
	dind=label1[ind];
	if (dind<10) llen=1;
	else if (dind>=  10 && dind<  100) llen=2;
	else if (dind>= 100 && dind< 1000) llen=3;
	else if (dind>=1000 && dind<10000) llen=4;
	sprintf(DefLabel,"%d", label1[ind]);
	XDrawImageString(dpy,pixmap,gc,rPix.x,rPix.y, DefLabel ,llen);
	
	DRAW_ARROW(rPix.x,rPix.y,
		   rPix.x+(int)nint((double)diamPix.x*effForce[ind].x/0.2),
		   rPix.y-(int)nint((double)diamPix.y*effForce[ind].y/0.2),
		   pixmap);
      }
    }
  }

  /*------------------------------------------------------*/
  
  /* draw a line frame around the simulation area */

  XSetForeground(dpy, gc, color[RED]);
  if (edgemin.x < simArea.x / 2.0)
    XDrawLine(dpy, pixmap, gc,
	      -(int)nint(scale.x * simArea.x / 2.0) + offsPix.x,
	      (int)nint(scale.y * simArea.y / 2.0) + offsPix.y + diamPix.y,
	      -(int)nint(scale.x * simArea.x / 2.0) + offsPix.x,
	      -(int)nint(scale.y * simArea.y / 2.0) + offsPix.y);
  if (edgemax.x > simArea.x / 2.0)
    XDrawLine(dpy, pixmap, gc,
	      (int)nint(scale.x * simArea.x / 2.0) + offsPix.x + diamPix.x,
	      (int)nint(scale.y * simArea.y / 2.0) + offsPix.y + diamPix.y,
	      (int)nint(scale.x * simArea.x / 2.0) + offsPix.x + diamPix.x,
	      -(int)nint(scale.y * simArea.y / 2.0) + offsPix.y);
  if (edgemin.y < simArea.y / 2.0)
    XDrawLine(dpy, pixmap, gc,
	      -(int)nint(scale.x * simArea.x / 2.0) + offsPix.x,
	      (int)nint(scale.y * simArea.y / 2.0) + offsPix.y + diamPix.y,
	      (int)nint(scale.x * simArea.x / 2.0) + offsPix.x + diamPix.x,
	      (int)nint(scale.y * simArea.y / 2.0) + offsPix.y + diamPix.y);
  if (edgemax.y > simArea.y / 2.0)
    XDrawLine(dpy, pixmap, gc,
	      -(int)nint(scale.x * simArea.x / 2.0) + offsPix.x,
	      -(int)nint(scale.y * simArea.y / 2.0) + offsPix.y,
	      (int)nint(scale.x * simArea.x / 2.0) + offsPix.x + diamPix.x,
	      -(int)nint(scale.y * simArea.y / 2.0) + offsPix.y);  
  
  if (wWinModFlag) {

    /* show a frame for a modification */
    offsPix.x = wDrawSize.x / 2 - (int)nint(scale.x * center.x);
    offsPix.y = wDrawSize.y / 2 + (int)nint(scale.y * center.y);
    crackc.x = scale.x * modarea.center.x + offsPix.x;
    crackc.y = -scale.y * modarea.center.y + offsPix.y;
    cracks.x = (double)(scale.x * modarea.size.x);
    cracks.y = (double)(scale.y * modarea.size.y);
    /*top-left corner*/
    crackp1.x = crackc.x+cos(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      -cos(modarea.angle*Pi/180)*cracks.x/2;
    crackp1.y = crackc.y-sin(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      +sin(modarea.angle*Pi/180)*cracks.x/2;
    /*bottom-left corner*/
    crackp3.x = crackc.x-cos(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      -cos(modarea.angle*Pi/180)*cracks.x/2;
    crackp3.y = crackc.y+sin(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      +sin(modarea.angle*Pi/180)*cracks.x/2;
    
    /*top-right corner*/
    crackp2.x = crackc.x+cos(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      +cos(modarea.angle*Pi/180)*cracks.x/2;
    crackp2.y = crackc.y-sin(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      -sin(modarea.angle*Pi/180)*cracks.x/2;
    /*bottom-right corner*/
    crackp4.x = crackc.x-cos(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      +cos(modarea.angle*Pi/180)*cracks.x/2;
    crackp4.y = crackc.y+sin(Pi/2+modarea.angle*Pi/180)*cracks.y/2
      -sin(modarea.angle*Pi/180)*cracks.x/2;
   
    XSetForeground(dpy, gc, color[BLACK]);

    XDrawLine(dpy, pixmap, gc, gtype(crackp1.x),gtype(crackp1.y),  
	      gtype(crackc.x),gtype(crackc.y));
    XDrawLine(dpy, pixmap, gc, gtype(crackp2.x),gtype(crackp2.y),  
	      gtype(crackc.x),gtype(crackc.y));
    
    XDrawLine(dpy, pixmap, gc, gtype(crackp1.x),gtype(crackp1.y), 
	      gtype(crackp3.x),gtype(crackp3.y));
    XDrawLine(dpy, pixmap, gc, gtype(crackp3.x),gtype(crackp3.y),  
	      gtype(crackp4.x),gtype(crackp4.y));
    XDrawLine(dpy, pixmap, gc, gtype(crackp4.x),gtype(crackp4.y),  
	      gtype(crackp2.x),gtype(crackp2.y));
    XDrawLine(dpy, pixmap, gc, gtype(crackp2.x),gtype(crackp2.y),  
	      gtype(crackp1.x),gtype(crackp1.y));		 
   
  }
  DrawColorBar();
  Refresh();

  

  /* update simulation text window */
  UpdateTextInfos(ctmpmin,ctmpmax);

  /* flush X queue */
  
  XSync(dpy, False);
}

/*------------------------------------------------------------*/
/*------------------------------------------------------------*/

/*
 * refresh display - copy pixmap to drawingarea window
 */
void Refresh()
{
  XCopyArea(dpy, rangepixmap, XtWindow(wRange),
	    gc, 0, 0, wRangeSize.x, wRangeSize.y, 0, 0);
  XCopyArea(dpy, pixmap, XtWindow(wDraw),
	    gc, 0, 0, wDrawSize.x, wDrawSize.y, 0, 0);
 
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*
 * refresh display - copy plotting area pixmap to window
 */
void RefreshPlot()
{
  XCopyArea(dpy, rangepixmap, XtWindow(wRange),
	    gc, 0, 0, wRangeSize.x, wRangeSize.y, 0, 0);
  XCopyArea(dpy, plpixmap, XtWindow(wPlot),
	    gc, 0, 0, wPlotSize.x, wPlotSize.y, 0, 0);
}

/*------------------------------------------------------------*/
/*------------------------------------------------------------*/

/*
 * Calculate colorindex based on the potential energy
 */
double calccolindex(double x, double xmin, double xmax) {
  static double xs,y;
  xs=(x-xmin)/(xmax-xmin);
  if (cix0>0 && ciy0>0) {
    if (xs<cix0) {
      y=xs/cix0*ciy0;
    } else {
      y=ciy0+(xs-cix0)/(1.0-cix0)*(1.0-ciy0);
    }
  }
  else
    y=xs;
  return y;
}

/*----------------------------------------------------------------*/
/*----------------------------------------------------------------*/

/*convert HSL to RGB (color codings)*/

void HSL_to_RGB(double h, double sl, double l, double *r, double *g, double *b)
{
  double v;
  v = (l <= 0.5) ? (l * (1.0 + sl)) : (l + sl - l * sl);
  if (v <= 0) {
    *r = *g = *b = 0.0;
  } 
  else {
    double m;
    double sv;
    int sextant;
    double fract, vsf, mid1, mid2;
    m = l + l - v;
    sv = (v - m ) / v;
    h *= 6.0;
    sextant = h;	
    fract = h - sextant;
    vsf = v * sv * fract;
    mid1 = m + vsf;
    mid2 = v - vsf;
    switch (sextant) {
    case 0: *r = v; *g = mid1; *b = m; break;
    case 1: *r = mid2; *g = v; *b = m; break;
    case 2: *r = m; *g = v; *b = mid1; break;
    case 3: *r = m; *g = mid2; *b = v; break;
    case 4: *r = mid1; *g = m; *b = v; break;
    case 5: *r = v; *g = m; *b = mid2; break;
    }
  }
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a text arrow widget and its parent HRowCol. The parameter parent
  is the higher-level parent, usually a form*/
Widget CreateRCTA(Widget parent, Widget upwid, char *label, int cbnum)
{
  Widget wRowCol2,arrow1,arrow2;
  wRowCol2=CreateHRowCol(parent,upwid);
  XtVaSetValues(wRowCol2,  XmNrightAttachment,        XmATTACH_FORM,NULL);

  switch (c[cbnum].nmdvalue) {
  case TAN:
    c[cbnum].text = CreateLabelText(wRowCol2, label, *c[cbnum].value, 
				    c[cbnum].length, ALLOW_NUMBERS, cbnum);
    break;
  case TAND:
    c[cbnum].text = CreateLabelText(wRowCol2, label, *c[cbnum].value, 
				    c[cbnum].length,
				    ALLOW_NUMBERS | ALLOW_DOT, cbnum);
    break;
  case TANMD:
    c[cbnum].text = CreateLabelText(wRowCol2, label, *c[cbnum].value, 
				    c[cbnum].length,
				    ALLOW_NUMBERS |  ALLOW_MINUS| ALLOW_DOT, 
				    cbnum);
    break;
  case TANM:
    c[cbnum].text = CreateLabelText(wRowCol2, label, *c[cbnum].value, 
				    c[cbnum].length,
				    ALLOW_NUMBERS | ALLOW_MINUS, cbnum);
    break;
  }
  arrow1 = XtVaCreateManagedWidget("",
				   xmArrowButtonWidgetClass,  wRowCol2,
				   XmNarrowDirection,         XmARROW_DOWN,
				   XmNmultiClick,             XmMULTICLICK_KEEP,
				   NULL);
  XtAddCallback(arrow1, XmNarmCallback, CbArrowTimerStart,
		(XtPointer)(long)(2 * cbnum));
  XtAddCallback(arrow1, XmNdisarmCallback, CbArrowTimerStart,
		(XtPointer)(long)(2 * cbnum));
  c[cbnum].arrow1=arrow1;
  
  arrow2 = XtVaCreateManagedWidget("",
				   xmArrowButtonWidgetClass,  wRowCol2,
				   XmNarrowDirection,         XmARROW_UP,
				   XmNmultiClick,             XmMULTICLICK_KEEP,
				   NULL);
  XtAddCallback(arrow2, XmNarmCallback, CbArrowTimerStart,
		(XtPointer)(long)(2 * cbnum + 1));
  XtAddCallback(arrow2, XmNdisarmCallback, CbArrowTimerStart,
		(XtPointer)(long)(2 * cbnum + 1));
  c[cbnum].arrow2=arrow2;
  
  return wRowCol2;
}
  

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/

/*
 * Create label and text to a parent row column
 */
Widget CreateLabelText(Widget parent, char *name, double value, int length,
			int allow, int cbNum)
{
  Widget text/*, arrow*helios*/;
  /*int i;*helios*/
  char buff[20];

  sprintf(buff, "%g", value);
  /* label */
  XtVaCreateManagedWidget(name, xmLabelWidgetClass, parent, NULL);

  text = XtVaCreateManagedWidget("Text widget",
    xmTextWidgetClass,         parent,
    XmNcolumns,                length,
    XmNrows,                   1,
    XmNmarginWidth,            4,
    XmNmarginHeight,           2,
    XmNcursorPositionVisible,  True,
    XmNeditable,               True,
    XmNeditMode,               XmSINGLE_LINE_EDIT,
    /*XmNmaxLength,              length,*/
    XmNvalue,                  buff,
    NULL);

  XtAddCallback(text, XmNmodifyVerifyCallback, CbTextModifyVerify,
                (XtPointer)(long)cbNum);
  XtAddCallback(text, XmNvalueChangedCallback, CbTextValueChanged,
                (XtPointer)(long)cbNum);
  XtAddCallback(text, XmNlosingFocusCallback, CbTextLoseFocus,
                (XtPointer)(long)cbNum);
/*
  for (i = 0; i < 2; i++) 
    arrow = XtVaCreateManagedWidget("",
      xmArrowButtonWidgetClass,  parent,
      XmNarrowDirection,         (i > 0) ? XmARROW_UP : XmARROW_DOWN,
      XmNmultiClick,             XmMULTICLICK_KEEP,
      NULL);
    XtAddCallback(arrow, XmNarmCallback, CbArrowTimerStart,
                  (XtPointer)(long)(2 * cbNum + i));
    XtAddCallback(arrow, XmNdisarmCallback, CbArrowTimerStart,
                  (XtPointer)(long)(2 * cbNum + i));
        
  }*/
  return text;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a new dialog window. Parent can be for example main window or
  a button in the main window. x and y are coordinates for the new dialog
  top-left corner. Activator is for a example a menu-item*/
Widget CreateDialog(Widget parent,char *title,int x,int y,Widget activator)
{
  Widget wDialog=XtVaCreatePopupShell (title,
    xmDialogShellWidgetClass, parent,
    XmNtitle,                 title,
    XmNnoResize,              True,
    XmNallowShellResize,      False,
    XmNx,                     x,
    XmNy,                     y,
    XmNminimizeButtons,       True, 
    NULL);
  XtAddEventHandler(wDialog,StructureNotifyMask,False,
		    (XtEventHandler) SizeMaster,NULL);
  XtAddCallback(wDialog,XmNpopdownCallback,CbCloseDialog,activator);
  return wDialog;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a basic horizontal rowcol. This is a suitable parent rowcol type
  for a text arrow widget. Topwid is the widget immediately above this.*/
Widget CreateHRowCol(Widget parent,Widget topwid)
{
 Widget wRowCol = XtVaCreateManagedWidget("RowColumn",
    xmRowColumnWidgetClass,    parent,
    XmNtopAttachment,          XmATTACH_WIDGET,
    XmNtopWidget,              topwid,
    XmNpacking,                XmPACK_TIGHT,
    XmNorientation,            XmHORIZONTAL,
    XmNmarginWidth,            1,
    XmNmarginHeight,           1,
    XmNspacing,                1,
    NULL);
 return wRowCol;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a basic vertical rowcol. This is a suitable general purpose manager
  widget which arranges child widgets vertically. This can be used for
  example inside a dialog sub-form or after a text arrow widget to arrange 
  widgets below it properly. However, this can NOT be a window/dialog main
  rowcol. Topwid is the widget immediately above this.*/ 
  
Widget CreateVRowCol(Widget parent,Widget topwid)
{
 Widget wRowCol = XtVaCreateManagedWidget("RowColumn",
    xmRowColumnWidgetClass,    parent,
    XmNtopAttachment,          XmATTACH_WIDGET,
    XmNtopWidget,              topwid,
    XmNpacking,                XmPACK_TIGHT,
    XmNorientation,            XmVERTICAL,
    XmNmarginWidth,            1,
    XmNmarginHeight,           1,
    XmNspacing,                1,
    NULL);
 return wRowCol;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* This rowcol is what the previous was not: a window/dialog main rowcol.
   This is usually used as general manager widget for a dialog/window. The 
   parent is usually a form or a pane whose parent is the dialog/window shell.
   This can also be used in other tasks but usually not.
*/

Widget CreateMainRowCol(Widget parent)
{
  Widget wRowCol = XtVaCreateManagedWidget("RowColumn",
    xmRowColumnWidgetClass,    parent,
    XmNtopAttachment,          XmATTACH_FORM,
    XmNleftAttachment,         XmATTACH_FORM,
    XmNbottomAttachment,       XmATTACH_FORM,
    XmNpacking,                XmPACK_TIGHT,
    XmNorientation,            XmVERTICAL,
    XmNspacing,                4,
    XmNmarginWidth,            2,
    XmNmarginHeight,           1,
    NULL);
  return wRowCol;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* This rowcol is used for a special task in the parameters and fit the 
   boundary dialogs. It creates a 2nd column to a window. It can also be
   used to create 3rd, 4th,..,and also the last column. The 1st column is 
   an ordinary mainrowcol. Leftwid arg of course specifies the left column.*/

Widget CreateRightRowCol(Widget parent,Widget leftwid)
{
  Widget wRowCol = XtVaCreateManagedWidget("RowColumn",
    xmRowColumnWidgetClass,    parent,
    XmNtopAttachment,          XmATTACH_FORM,
    XmNleftAttachment,         XmATTACH_WIDGET,
    XmNleftWidget,             leftwid,
    XmNbottomAttachment,       XmATTACH_FORM,
    XmNpacking,                XmPACK_TIGHT,
    XmNorientation,            XmVERTICAL,
    XmNspacing,                4,
    XmNmarginWidth,            2,
    XmNmarginHeight,           1,
    NULL);
  return wRowCol;
}

/* This was the last pre-defined rowcol type. If you need a special rowcol
   you can copy-paste one of these and modify it. If you only need a few 
   special rowcols, you can just put them directly to dialogs without making
   a Create...RowCol function. But if you need several of them, you should
   consider making a creating function.*/

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*This creates a scale like "Atom diam %". Args are: parent (usually
  vrowcol), minimum value, maximum value, initial value, label, callback
  macro value (see control.h).*/

Widget CreateScale(Widget parent,int min,int max,int val,char *label,int cb)
{
  Widget wScaleDiam = XtVaCreateManagedWidget(label,
    xmScaleWidgetClass,        parent,
    XmNorientation,            XmHORIZONTAL,
    XmNshowValue,              True, 
    XmNminimum,                min,
    XmNmaximum,                max,
    XmNvalue,                  val,
    XtVaTypedArg,              XmNtitleString, T_STRING(label),
    NULL);
  XtAddCallback(wScaleDiam, XmNvalueChangedCallback, CbScale, 
		(XtPointer)cb);
  return wScaleDiam;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*This creates a frame. Usually used in dialogs to group widgets in a visible
  frame (form/pane doesn't show boundaries).*/

Widget CreateFrame(Widget parent)
{
 Widget wFrame = XtVaCreateManagedWidget("frame",
    xmFrameWidgetClass,        parent,
    XmNshadowType,             XmSHADOW_ETCHED_IN,
    XmNspacing,                1,
    XmNmarginWidth,            1,
    XmNmarginHeight,           1,
    NULL);
 return wFrame;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*This creates a form which is used as a manager widget almost everywhere.
  Parent is often a main window or a vrowcol class widget. Margin specifies
  how much empty space is left around widgets in the form.*/

Widget CreateForm(Widget parent,int margin)
{
  Widget wForm = XtVaCreateManagedWidget("form",
    xmFormWidgetClass,         parent,
    XmNmarginWidth,            margin,
    XmNmarginHeight,           margin,
    NULL);
  return wForm;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*This creates a pane which is used as a main manager widget is dialogs. No
  other tasks currently. Parent: class dialogshell.*/

Widget CreatePane(Widget parent)
{
  Widget wPane=XtVaCreateWidget("pane",
    xmPanedWindowWidgetClass,parent,
    XmNsashWidth,1,XmNsashHeight,1,
    NULL);
  return wPane;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a label which shows the string *label.*/

Widget CreateLabel(Widget parent, char *label)
{
  Widget wLabel = XtVaCreateManagedWidget(label,
    xmLabelWidgetClass,        parent,
   NULL);
  return wLabel;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* Creates a textinfo like the one in top-left corner in the main window.*/
Widget CreateTextInfo(Widget parent,int cols,int rows,int mw,int mh)
{
  Widget wTextInfo = XtVaCreateManagedWidget("Text widget",
    xmTextWidgetClass,         parent,
    XmNcolumns,                cols,
    XmNrows,                   rows,
    XmNmarginWidth,            mw,
    XmNmarginHeight,           mh,
    XmNcursorPositionVisible,  False,
    XmNeditable,               False,
    XmNeditMode,               XmMULTI_LINE_EDIT,
    NULL);
  return wTextInfo;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* Creates an option menu or a pulldown menu. This can not create a menubar.
   Parent is usually a menubar for pulldown menus and vrowcol for option menus.
   Mnemonic is short-cut to open the menu. See graphvar.h for *items.*/
Widget CreateMenu(Widget parent,int menu_type,char *menu_title,
		  char menu_mnemonic,MenuItem *items)
{
  Widget menu, cascade, widget;
  int i;
  XmString str;
 
  menu = XmCreatePulldownMenu(parent,"_pulldown",NULL,0);
  
  str = XmStringCreateLocalized (menu_title);
  if (menu_type == XmMENU_PULLDOWN) {
    cascade = XtVaCreateManagedWidget (menu_title,
       xmCascadeButtonGadgetClass,parent,
       XmNsubMenuId,menu,
       XmNlabelString,str,
       XmNmnemonic,menu_mnemonic,
       NULL);	

  }		
  else {
    Arg args[5];
    int n = 0;
    XtSetArg (args[n],XmNsubMenuId,menu);n++;
    XtSetArg (args[n],XmNlabelString,str);n++;
    cascade = XmCreateOptionMenu (parent,menu_title,args,n);
  }
  XmStringFree(str);

  for (i=0;items[i].label != NULL; i++) {
    widget = XtVaCreateManagedWidget (items[i].label,*items[i].class,
				      menu,NULL);
    if (items[i].mnemonic)
      XtVaSetValues(widget,XmNmnemonic,items[i].mnemonic,NULL);
    if (items[i].accelerator) {
      str = XmStringCreateLocalized(items[i].accel_text);
      XtVaSetValues (widget,
		     XmNaccelerator,items[i].accelerator,
		     XmNacceleratorText,str,
		     NULL);
      XmStringFree (str);
    }

    if (items[i].callback)
      XtAddCallback (widget,
		     (items[i].class == &xmToggleButtonWidgetClass ||
		      items[i].class == &xmToggleButtonGadgetClass) ?
		     XmNvalueChangedCallback : /*togglebutton*/
		     XmNactivateCallback,      /*pushbutton*/
		     items[i].callback,(XtPointer)items[i].callback_data);
    switch (items[i].callback_data) {
    case M_NEW:
      wmNew = widget;
      break;
    case M_TEMPERATURE:
      wmTemp = widget;
      break;
    }
  }
  return cascade;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a togglebutton. Flag is the variable associated with this widget,
  cbdata is callback macro number (see control.h).*/

Widget CreateToggleB(Widget parent,char *label,int flag,int cbdata)
{
  Widget wToggle = XtVaCreateManagedWidget(label,
    xmToggleButtonWidgetClass, parent,
    XmNset,                    flag,
    NULL);
  XtAddCallback(wToggle, XmNvalueChangedCallback, CbButton,(XtPointer)cbdata);
  return wToggle;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/*Creates a pushbutton. Label is the button text, cbdata is callback macro 
  number (see control.h).*/
Widget CreatePushB(Widget parent,char *label,int cbdata)
{
  Widget wPushB = XtVaCreateManagedWidget(label,
    xmPushButtonWidgetClass, parent,
    XmNalignment,XmALIGNMENT_CENTER,
    XtVaTypedArg, XmNlabelString, T_STRING(label),	      
    NULL);
  XtAddCallback(wPushB, XmNactivateCallback, CbButton,(XtPointer)cbdata);
  XtVaSetValues(wPushB, XmNalignment,XmALIGNMENT_CENTER, NULL);
  return wPushB;
}

/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/

/* this function prevents a shell from resizing itself*/
  
void SizeMaster(Widget shell,XtPointer client_data,XEvent *event)
{
  XConfigureEvent *cevent = (XConfigureEvent *) event;
  
  if (cevent->type != ConfigureNotify) return;
  XtVaSetValues(shell,
		XmNmaxWidth,cevent->width,
		XmNminWidth,cevent->width,
		XmNmaxHeight,cevent->height,
		XmNminHeight,cevent->height,
		NULL);
  XtRemoveEventHandler(shell,StructureNotifyMask,False,
		       (XtEventHandler) SizeMaster,NULL);
}
