
#ifdef MAINFILE
#define DEF 
#else
#define DEF extern
#endif


/***************************************************************************/

/* Simulation parameters, setup and running */

DEF double Pi;
DEF int sighupFlag;       /* signal to take a snapshot using the kill command */ 
DEF long int seed;        /* AK: seed for random number generator  */
DEF Widget wParamMan;     /* parameters window main manager widget */   
DEF Widget wPushBOK;      /* OK widget (in parameters window)      */ 
DEF Widget wPushBCancel;  /* Cancel button                */
DEF Widget wmNew;         /* menu item "new"              */

/* Sim. area */
DEF VecD size;           /* sim. area size                */
DEF VecD simArea;         /* simulation area              */
DEF double yMin,yMax,xMin,xMax; /* max and min coordinates of atoms*/ 

/* Lattice params etc */
DEF double epsilon;       /* e1 potential parameter       */
DEF double epsilon2;      /* e2                           */
DEF double lParam;        /* l1 lattice parameter         */
DEF double lParam2;       /* l2                           */
DEF double relaxation;    /* lattice relaxations          */
DEF double cut;           /* cutoff range factor          */

/* Modifications to atom configuration */
DEF int boundaryFlag;     /* boundary on/off              */
DEF int verticalFlag;     /* vertical boundary            */ 
DEF int suggestFlag;      /* fixing suggestion            */
DEF double borderW;       /* width of fixed borders       */

/* Seaming -related variables */
DEF Widget wSeamMan;      /* seaming window main manager  */ 
DEF int wWinSeamFlag;     /* true when the dialog is open */
DEF Seamdata seam;        /* seaming parameters           */
DEF Widget wSeamPushB;    /* seaming -button              */

/* Periodic boundary conditions. AK Oct 2000 */
DEF int periodicXFlag;    /* periodic borders in x dir    */
DEF VecD simArea2;        /* Half of simArea              */
DEF Widget wPerxToggle;   /* periodicX toggle button      */

/* Crack */
DEF RArea crack;
DEF int crackFlag;

/* Simulation progress */
DEF double deltaT;        /* time step in calculations    */
DEF int updateStep;       /* graphics update step         */
DEF long int stepCount;   /* current simulation step      */

/* Running controls */
DEF Widget wRunPushB;     /* run button                   */
DEF int runningFlag;      /* program on/off state         */
DEF char xyzloadnumber[3];/*used in atom configuration loading*/
DEF int autostartFlag;
DEF int readatoms;

/* Simulation data storages */
DEF int *atype;           /* identifying the atom type    */ 
DEF int nAtom;            /* current number of atoms      */
DEF int *fixvel;          /* atomies that have fixed velocities       */
DEF VecD *cforce;         /* constant force (modify-dialog)           */      
DEF double forcemin,forcemax; /* min/max total force      */
DEF VecD *ra;             /* accelerations of the atoms   */
DEF int nAtomOrig;        /* original number of atoms     */
DEF int *borderatom;      /* if >0 atom is on the border (see simu.h) */
DEF VecD *rv;             /* velocities of the atoms      */
DEF VecD *r;              /* positions of the atoms       */         
DEF VecI cells;           /* number of atom cells         */
DEF int *cellList;        /* array for linked list of cells           */
DEF double potEnmin ,potEnmax,potEnergy,*potEn; /* potential energy   */ 
DEF double kinEnmin,kinEnmax,kinEnergy;         /* kinetic energy     */

/* Tscheme (old name, now .event but internally still tscheme*/
DEF int iTscheme;         /* event number                 */
DEF char tfname[MAXLINE]; /* loaded. event file name      */
DEF int tadjustFlag;      /* adjust .event positions      */
DEF int tapplyFlag;       /* use the selected .event      */
DEF int tadjust2Flag;     /* adjust .event sizes          */
DEF int savetscheme;      /* save .event file             */
DEF FILE *tsavefile;      /* file save.event              */

/*************************************************************************/

/*Area modifier*/
DEF RArea modarea;        /* modify area selected area    */ 
DEF Widget wModMan;       /* modify main manager widget   */
DEF int wWinModFlag;      /* modify dialog visible/univis */
DEF int modinfo;          /* modification type (fix etc)  */
DEF VecD modvel;          /* some mod. types use this     */
DEF int showselFlag;      /* flag for show selection      */
          


/*************************************************************************/

/*Strain, stress and pressure*/
DEF Widget wStrainMan;    /* pressure dialog manager      */
DEF Widget wFixedToggle;  /* fixed borders checkbox (obs. */
DEF Widget wStrainPushB;  /* apply strain button (obs.)   */
DEF int fixedFlag;        /* obsolete                     */
DEF int strainFlag;       /* obsolte                      */
DEF double strainStep;    /* total strain per cent (obs.) */
DEF double strainP;       /* strain % (obs.)              */
DEF double strainT;       /* strain t (obs.)              */
DEF double pressure;      

DEF double stressijmin,stressijmax,*stressij; /* stresses */

/**************************************************************************/

/* Temperature control */
DEF Widget wTempMan;      /* temperature window manager   */
DEF Widget wmTemp;        /* temperature menu item        */
DEF CoolData cool;        /* temperature control params   */

DEF Widget wLowTempToggle;/* toggle                       */
DEF Widget wQuenchToggle; /* toggle                       */
DEF Widget wCoolPushB;    /* start cooling                */

/*************************************************************************/

/* Tracking procedure */
DEF int FileFlag;
DEF int StartFlag;       
DEF int wWinTrackFlag;   /* tracking window vis/unvis     */
DEF int SetTrackFlag;    /* defect tracking on/off        */
DEF int showTrackFlag;   /* show defect positions         */
DEF int saveTrackFlag;   /* save track on/off             */
DEF double PotFact;      /* potential energy param        */
DEF double RRc;          /* range param                   */
DEF double BFactor;      /* border param                  */
DEF int NumberofDefects; 
DEF VecD *rb,*rTrk,*effForce;
DEF Widget wTrackMan,wSaveTrackToggle,wShowTrackToggle;
DEF int label1[MAXLABELS],numero[5];

/*************************************************************************/

/* Animation */
DEF Widget wAniMan;      /* animation manager widget      */
DEF AniData ani;         /* animation parameters          */
DEF Widget wEndendToggle,wAniPushB; /*buttons             */
DEF int wWinAniFlag;     /* ani window is open/closed     */

/*************************************************************************/

/*Variables mostly associated with drawing, not computation*/



/*color coding*/
DEF ColorRanges colorranges; /* all colorrange params     */
DEF Widget wColorMan;    /* color ranges main manager     */

DEF int sinfo;           /* display pot/kin energy etc    */

DEF Widget wZoomMan;     /* zoom window manager           */  
DEF Widget wZoomScale;   /* zooming scale bar widget      */        

/*Averaging vectors*/
DEF VecD *avr,*avv;      /* positions and velocities      */   
DEF int *avn,*avatype;   /* numbers and types             */
DEF double avrCut;       /* averaging value               */   
DEF VecI avCells;        /* number of avr. cells          */    

/*display variables*/
DEF VecI wDrawSize;      /* size of the main drawing area */
DEF VecD zoom;           /* zoom.x and zoom.y             */
DEF VecD center;         /* center                        */
DEF VecD edge;           /* edges (red lines)             */

DEF XtAppContext app;    /* application context, Xt data  */
DEF Display *dpy;        /* display struct                */

DEF Widget wDraw;        /* Top widget for getting window id */
DEF Widget wHScrollB;    /* atom area horiz.scrollbar     */
DEF Widget wVScrollB;    /* atom area vertic.scrollbar    */
DEF double diam;         /* atom diam                     */

/* Plotting */
DEF Plotdata plotdata;   /* plotting parameters           */

DEF int printe;          /* if this is >0, print total kinetic and potential 
			    energy at intervals of printe */

/*************************************************************************/

#undef DEF
