#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>  /*DBL_MAX*/ 
#include <float.h>   /*DBL_MAX*/

#include "control.h"
#include "graphics.h"
#include "simu.h"
#include "fitsigma.h"
#include "calc.h"
#include "callback.h"
#include "gvar.h"    /*load global variables*/
#include "track.h"

/*Definitions used in simu.c.*/

/* default values */
#define SMALLTEMP 1.0e-15
#define MAXTYPE 3

#ifdef ONEBODY
/* One-body energies and forces */
double OneBody(double x, double y, double *e, double *fx, double *fy);
#endif

double Energy;
double eps[MAXTYPE][MAXTYPE] = {{0,0,0},{0,0,0},{0,0,0}};
double cutoffdist[MAXTYPE][MAXTYPE]={{0,0,0},{0,0,0},{0,0,0}};
double sig[MAXTYPE][MAXTYPE]={{0,0,0},{0,0,0},{0,0,0}};   
#ifdef HARMONIC
double r0[MAXTYPE][MAXTYPE]={{0,0,0},{0,0,0},{0,0,0}};   
double kharm[MAXTYPE][MAXTYPE]={{0,0,0},{0,0,0},{0,0,0}};   
#endif
double temperature;
int *bottom, *top;           /* arrays for fixed atoms */
int borderNMax;            /* size of bottom[] and top[] */
MtrxD *StressT = NULL;  /*Stress Tensor...*/
int strainCount = 0;

extern Container c[];

/*--------------------------------------------------------------------*/
/*--------------------------------------------------------------------*/


/*
 * initialize computation
 */
void InitRun()
{
  int i;
#ifdef HARMONIC
  double r26;
  double ktmp;
#endif
  double lpar;
  double rCut,rCutB,rCutAB;/* time step and potential cutoff distances */
  double  sigma,sigma2;    /* potential normalizing parameters */
  kinEnergy = 1.0;
  /* defaults */
  temperature =TEMPERATURE;
  

  /* Cut off distances in different cases AA BB AB and BA */
  rCut = cut *lParam / sqrt(2.0);
  rCutB = cut *lParam2/ sqrt(2.0);
  rCutAB = pow(((SQ4(rCut)*SQ(rCut)+SQ4(rCutB)*SQ(rCutB))/2),(1.0/6.0));
  cutoffdist[1][1]= rCut;
  cutoffdist[2][2]=rCutB;
  cutoffdist[1][2]=  rCutAB;
  cutoffdist[2][1]= cutoffdist[1][2];


  /*Potential parameters*/

  sigma =  pow(0.50*(pow(cut,-7.0)-1.0)/(pow(cut,-13.0)-1.0),
	       1.0/6.0)*lParam/sqrt(2.0);
  sigma2 = pow(0.50*(pow(cut,-7.0)-1.0)/(pow(cut,-13.0)-1.0),
	       1.0/6.0)*lParam2/sqrt(2.0);
  /*fprintf(stderr,"lParam %.12g %.12g\n",lParam,lParam2);
  fprintf(stderr,"sigma  %.12g %.12g\n",sigma,sigma2);*/
  sigma=fitsigma(lParam/sqrt(2.0),rCut);
  sigma2=fitsigma(lParam2/sqrt(2.0),rCutB);
  sigma  *= 1.0/sqrt(2.0)/0.6999;  /* For some reason sigma has to be 
				      scaled up a bit!!!!! */
  sigma2 *= 1.0/sqrt(2.0)/0.6999;  /* For some reason sigma has to be 
				      scaled up a bit!!!!! */
  /*fprintf(stderr,"sigma  %.12g %.12g\n",sigma,sigma2);*/
  sig[1][2]=pow(((SQ4(sigma)*SQ(sigma)+SQ4(sigma2)*SQ(sigma2))/2),(1.0/6.0));
  sig[2][1] = sig[1][2];
  sig[1][1]=sigma;
  sig[2][2]=sigma2;
  
  eps[1][1] = epsilon;
  eps[2][2] = epsilon2;
  eps[1][2]=(2*sqrt(epsilon*epsilon2)*SQ(sigma)*sigma*sigma2*SQ(sigma2))/
    ((SQ4(sigma)*SQ(sigma)+SQ4(sigma2)*SQ(sigma2)));
  eps[2][1] = eps[1][2];

#ifdef HARMONIC
  /* Equilibrium position (used in harmonic potential) */
  r26=pow(2.0,1.0/6.0);
  r0[1][1]=r26*sig[1][1];
  r0[1][2]=r26*sig[1][2];
  r0[2][1]=r26*sig[2][1];
  r0[2][2]=r26*sig[2][2];
  /* Harmonic potential force constants */
  ktmp=9.0/pow(2.0,1.0/3.0);
  kharm[1][1]=ktmp*UNITENERGY*eps[1][1]/pow(sig[1][1],2.0);
  kharm[1][2]=ktmp*UNITENERGY*eps[1][2]/pow(sig[1][2],2.0);
  kharm[2][1]=ktmp*UNITENERGY*eps[2][1]/pow(sig[2][1],2.0);
  kharm[2][2]=ktmp*UNITENERGY*eps[2][2]/pow(sig[2][2],2.0);
#endif
  
  nAtomOrig = nAtom;
  if (rCut>=rCutB){
    VSCopy(cells, 1.0 / rCut, simArea);
  }
  else{
    VSCopy(cells, 1.0 / rCutB, simArea);
  }
  if (avrCut > 0.099)
    VSCopy(avCells, 1.0 / avrCut, simArea);
  else {
    avCells.x = 0;
    avCells.y = 0;
  }
  
  /* memory (re)allocation */
  ALLOC(rv, nAtom, VecD);
  ALLOC(ra, nAtom, VecD);
  ALLOC(cforce,nAtom,VecD);
  ALLOC(cellList, nAtom + VProd(cells), int);
  ALLOC(avv, VProd(avCells), VecD);
    
  /*border atom max count calculations*/
  if (lParam<lParam2)
    lpar=lParam;
  else
    lpar=lParam2;
  borderNMax = (int) ceil(3*2*size.x/(lpar/sqrt(2.0)));
  ALLOC(bottom, borderNMax, int);
  ALLOC(top, borderNMax, int);
  
  ALLOC(potEn,nAtomOrig,double);
  ALLOC(stressij,nAtomOrig,double);

  /*---Stress--*/
  ALLOC(StressT,nAtom,MtrxD);

  /* determine maximum and minimum coordinates of the atoms */

  xMin = 0.0;
  xMax = 0.0;
  yMin = 0.0;
  yMax = 0.0;
  
  for (i = 0; i < nAtom; i++) {
    cforce[i].x=0.0;cforce[i].y=0.0;
    if (r[i].x < xMin)      xMin = r[i].x;
    else if (r[i].x > xMax) xMax = r[i].x;
    else if (r[i].y < yMin) yMin = r[i].y;
    else if (r[i].y > yMax) yMax = r[i].y;
  }

  /* initialize computation */
  InitialState();
  ComputeForces();
  CalcEnergy();

  /* set flags */
  /*changeTemp = 0;*/
  stepCount = 0;
  strainFlag = 0;
  iTscheme=0;
  strainCount = 0;
}


/*---------------------------------------------------*/
/*---------------------------------------------------*/


/*
 * work procedure: single md step
 */

Boolean MdCycle()
{
 
  int i,updstep;
  double newtemp,currenttime;


  /* fprintf(stderr,"MdCycle %d %d\n",stepCount,printe); */

  /*if (progFlag==0) return False;*not in use?*/  
  if (nAtom == 0) return False;
  ++stepCount;
  if (tapplyFlag) checkTscheme();
  ComputeForces();
  if (nAtom == 0) return False;

  LeapfrogIntegrate();
  CalcEnergy();
  
  if (strainFlag) { /* apply strain */  

    if (strainCount>=(int)(strainT/(1000*deltaT)) ||
       (strainStep>0 && (xMax>simArea.x/2.0-strainStep || 
			 xMin<-simArea.x/2.0+strainStep)) || 
	(strainStep<0 && xMax-xMin<4*borderW)) {
      strainFlag = 0;
      strainCount = 0;
      XtSetSensitive(wStrainPushB, True);
      XtSetSensitive(wFixedToggle, True);
      TASensitive(STRAIN_T, True);
      TASensitive(STRAIN_P, True);

    } else {
      
      xMin=yMin=+1e10;
      xMax=yMax=-1e10;
      for (i = 0; i <nAtom ; i++) {
	if (borderatom[i] & /* BORDER_TOP */ (BORDER_LEFT|BORDER_RIGHT)) 
	  r[i].x += strainStep /* *r[i].x */ /20;
	if (r[i].x < xMin)      xMin = r[i].x;
	else if (r[i].x > xMax) xMax = r[i].x;
	if (r[i].y < yMin)      yMin = r[i].y;
	else if (r[i].y > yMax) yMax = r[i].y;
      }
      strainCount++;
    }
  }
  
  /* Temperature */ 
  

  currenttime=deltaT*stepCount*1000.0;

  if (cool.programon) {
    if (currenttime<cool.starttime) 
      SetNewTemp(cool.cooltemp/temperature);
    else {
      newtemp=cool.cooltemp + (cool.slope*(currenttime-cool.starttime));
      SetNewTemp(newtemp/temperature);
    }
  } else {
    if (cool.TScaleFlag) SetNewTemp(cool.tempvalue/temperature);
  }
    
    
  updstep=(int)((double)updateStep/(1000.0*deltaT)+0.5);
  if (updstep<1) updstep=1;
  if (stepCount % updstep == 0 ) {
    Redraw();
    RedrawPlot(0); 
  }
  
  if (sighupFlag) {
    SnapShot();
    sighupFlag=0;
  }

  if (printe>0) {
    if (stepCount % printe == 0) {
      fprintf(stdout,"%.15g %.15g %.15g %d\n",
	      (double)stepCount*1000.0*deltaT,kinEnergy,potEnergy,stepCount);
      fflush(stdout);
    }
  }

  if (ani.aniFlag && ani.snaphelp<(stepCount * 1000 * deltaT)){
    if (!ani.limitsFlag){
      SnapShot();
      ani.snaphelp += ani.waittime;
    } else {
      if (ani.snaphelp<ani.beginat){
	ani.snaphelp = ani.beginat;
      } else if ((stepCount * 1000 * deltaT-ani.waittime/10)<=ani.endat) {
	SnapShot();
	ani.snaphelp += ani.waittime;
      } else if (!ani.endendFlag) {
	ani.aniFlag = False;
      } else {
	bexit(0);
      }
    }
  }
  
  /*------------------Adding Tracking-----------------*/
  if (SetTrackFlag){
    Trackprocedure();
    if (saveTrackFlag) 
      savingprogram();
  };
  /*---------------------------------------------------*/

  return False;
}


/*--------------------------------------------------------------*/
/*--------------------------------------------------------------*/


/*
 * compute force and potential energy using cell method. The heart of the 
 * program.
 */
/* convenience macros */

#define DO_ATOM for (n = 0; n < nAtom; n ++)
#define DO_CELL(j, m) for (j = cellList[m]; j >= 0; j = cellList[j])

void ComputeForces()
{
  VecI cc, m1v, m2v, iof[] = {{0,0}, {1,0}, {1,1}, {0,1}, {-1,1}};
  VecD dr, invWid, rs;
  double ri2[MAXTYPE][MAXTYPE]={{0,0,0},{0,0,0},{0,0,0}};
  double ri6[MAXTYPE][MAXTYPE]={{0,0,0},{0,0,0},{0,0,0}};
  double urc[MAXTYPE][MAXTYPE],durc[MAXTYPE][MAXTYPE];
  double fcVal, r2,sigma6,rang2,rang6;
  int j1, j2, offset, btot, ttot,i,j;
  int help1,help2;
 
  VecI cellmin,cellmax;
  double dtmp;
#ifdef HARMONIC
  double rtmp;
#endif
  double stress;
  unsigned int c, m1, m2, n;

#ifdef ONEBODY
  /* For one-body energies and forces */
  double e1,f1x,f1y;
#endif
  
  /* This loop calculates Lennart-Jones potential at cutoff distance (urc) */
  /* and then the derivative (durc) of potential, in every possible case.  */

  for(i=0;i<MAXTYPE;i++){
    for(j=0;j<MAXTYPE;j++){
      dtmp=sig[i][j];
      sigma6 = dtmp*dtmp*dtmp*dtmp*dtmp*dtmp;
      if (SQ(cutoffdist[i][j])!=0){
	ri2[i][j]=1.0/SQ(cutoffdist[i][j]);
	ri6[i][j]= ri2[i][j] * ri2[i][j] * ri2[i][j];
	urc[i][j] = UNITENERGY*eps[i][j]*sigma6*ri6[i][j]
	  *(sigma6*ri6[i][j]-1.0);
	durc[i][j] = 12.0*UNITENERGY*eps[i][j]
	  *sigma6*ri6[i][j]*(0.5-sigma6*ri6[i][j])/cutoffdist[i][j];  
      }
    }
  }
  

  simArea2.x=simArea.x/2.0;
  simArea2.y=simArea.y/2.0;
  btot = 0;
  ttot = 0;
  
  /* initialize linked lists */

  for (n = nAtomOrig; n < nAtomOrig + VProd(cells); n++) cellList[n] = -1;
  VDiv(invWid, cells, simArea);

  /* assign atoms to cells */

  for (n = 0; n < nAtom; n++) {
    potEn[n]=stressij[n]=0.0;
    /* remove atoms that get outside the cell area */
    if (fabs(r[n].x) > 0.5 * simArea.x || fabs(r[n].y) > 0.5 * simArea.y) {
      r[n].x=r[--nAtom].x;
      r[n].y=r[nAtom].y;
      rv[n]=rv[nAtom];
      ra[n]=ra[nAtom];
      atype[n]=atype[nAtom];
      fixvel[n]=fixvel[nAtom];
      borderatom[n]=borderatom[nAtom];
    } 
    else { /* atom inside cell area */
      VSAdd(rs, r[n], 0.5, simArea);
      /* assign atom to a simulation cell */
      VMul(cc, rs, invWid);
      c = VLinear(cc, cells) + nAtomOrig;
      cellList[n] = cellList[c];
      cellList[c] = n;
    }
  }

  /* Cell limits depend on boundary conditions */
  if (periodicXFlag) {
    cellmin.x=-1;
    cellmax.x=cells.x+1;
  } else {
    cellmin.x=0;
    cellmax.x=cells.x;
  }    
  cellmin.y=0;
  cellmax.y=cells.y;
      
  
  /* zero accelerations */
  DO_ATOM {
    /*if (cforce[n].x != 0.0 || cforce[n].y != 0.0) {*/
    ra[n].x=cforce[n].x*1.0;
    ra[n].y=cforce[n].y*1.0;
    /*}
    else
      VZero(ra[n]);*/
    VZero(StressT[n].x);
    VZero(StressT[n].y);
  }
  potEnergy = 0.0;
  /* loop over all cells */
  for (m1v.y = 0; m1v.y < cells.y; m1v.y++) {
    for (m1v.x = 0; m1v.x < cells.x; m1v.x++) {
      m1 = m1v.y * cells.x + m1v.x + nAtomOrig;
      /* loop over offsets between adjacent cells */
      for (offset = 0; offset < 5; offset ++) {
	VAdd(m2v, m1v, iof[offset]);
	/* check that the cell inside the simulation area */
	if (m2v.x>=cellmin.x && m2v.x<cellmax.x 
	    && m2v.y>=cellmin.y && m2v.y<cellmax.y) {
	  /* Handle periodicity */
	  if (periodicXFlag) {
	    if (m2v.x<0) m2v.x=cells.x-1;
	    if (m2v.x>=cells.x) m2v.x=0;
	  }
	  
	  m2 = VLinear(m2v, cells) + nAtomOrig;
	  /* loops over atoms in each cell */
	  DO_CELL (j1, m1) {
	    DO_CELL (j2, m2) {
	      /* ensure each atom pair examined only once */
	      if (m1 != m2 || j2 < j1) {
		VSub(dr, r[j1], r[j2]);
		if (periodicXFlag) {
		  if (dr.x<-simArea2.x) dr.x+=simArea.x;
		  if (dr.x>=simArea2.x) dr.x-=simArea.x;
		}
		
		r2 = VLenSq(dr);
		/* if atoms within interaction range */
		help1=(int)fabs((double)atype[j1]);
		help2=(int)fabs((double)atype[j2]);
		if (r2 <SQ(cutoffdist[help1][help2])) {
#ifdef HARMONIC
		  rtmp=r0[help1][help2]-sqrt(r2);
		  fcVal=2.0*kharm[help1][help2]*rtmp/sqrt(r2);
		  Energy=kharm[help1][help2]*rtmp*rtmp;
#else
		  rang2 = 1.0 / r2;
		  rang6 = rang2 * rang2 * rang2;
		  dtmp=sig[help1][help2];
		  dtmp=dtmp*dtmp*dtmp*dtmp*dtmp*dtmp;
		  
		  fcVal = 12.0*UNITENERGY*eps[help1][help2]*dtmp
		    *rang6*(dtmp*rang6-0.5)*rang2
		    +durc[help1][help2]*sqrt(rang2);  
		  
		  /* update accelerations and energy sum */
		  Energy = UNITENERGY*eps[help1][help2]*dtmp*rang6
		    *(dtmp*rang6-1.0)+
		    (cutoffdist[help1][help2]-sqrt(r2))*
		    durc[help1][help2]-urc[help1][help2];
#endif		  
		  VVSAdd(ra[j1], fcVal, dr);
		  VVSAdd(ra[j2], - fcVal, dr);
		  if (sinfo == SXX || sinfo == SXY || sinfo == SYY) {
		  /*if (showxx||showxy||showyy) {*/
		    if (sinfo == SXX) stress=fcVal*dr.x*dr.x;
		    if (sinfo == SXY) stress=fcVal*dr.x*dr.y;
		    if (sinfo == SYY) stress=fcVal*dr.y*dr.y;
		    stressij[j1]+=stress/2.0;
		    stressij[j2]+=stress/2.0;
		  } 
		  potEn[j1]+=Energy/2.0;
		  potEn[j2]+=Energy/2.0;
		  potEnergy +=Energy;
		  
		}
	      }
	    } /* DO_CELL (j2, m2) */
	    /*test m1...*/
	    if (pressure != 0.0 && m1 == m2) { /* search edge atoms */
	      if (r[j1].y < yMin + borderW && r[j1].y < 0 
		  /*&& btot < borderNMax*/) {
		bottom[btot++] = j1;
	      } 
	      else if (r[j1].y > yMax - borderW && r[j1].y > 0 
		       /*&& ttot < borderNMax*/) {
		top[ttot++] = j1;
 	      }
	    } /* if (m1 == m2 */
	  } /* DO_CELL (j1, m1) */
	} /* if (m2v.x >= 0 ... */
      } /* for (offset = 0 ... */
    } /* for (m1x = 0 ... */
  } /* for (m1y = 0 ... */

#ifdef ONEBODY
  /* Add one-body energy and force */
  for (n = 0; n < nAtom; n++) {
    OneBody(r[n].x,r[n].y,&e1,&f1x,&f1y);
    potEn[n]+=e1;
    potEnergy+=e1;
    ra[n].x+=f1x;
    ra[n].y+=f1y;
  }
#endif


  /* Apply pressure */ 
  if (pressure != 0.0 && btot != 0 && ttot != 0) {
    fcVal = pressure * (xMax - xMin) / btot;
    for (j1 = 0; j1 < btot; ++j1) ra[bottom[j1]].y += fcVal;
    fcVal = pressure * (xMax - xMin) / ttot;
    for (j1 = 0; j1 < ttot; ++j1) ra[top[j1]].y -= fcVal;
  }
  
  potEnmin=1.0e20;
  potEnmax=-1.0e20;
   if (sinfo == SXX || sinfo == SXY || sinfo == SYY) {
     /*if (showxx||showxy||showyy) {*/
    stressijmin=1.0e20;
    stressijmax=-1.0e20;
  }
  for (n = 0; n < nAtom; n++) {
    if (potEn[n]<potEnmin) potEnmin=potEn[n];
    if (potEn[n]>potEnmax) potEnmax=potEn[n];
     if (sinfo == SXX || sinfo == SXY || sinfo == SYY) {
    /*if (showxx||showxy||showyy) {*/
      if (stressij[n]<stressijmin) stressijmin=stressij[n];
      if (stressij[n]>stressijmax) stressijmax=stressij[n];
    }
  }
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*
 * Averaging vectors: calculate avarage of rArg
 */
void Average(VecD *rArg)
{
  int i, c;
  VecD invWid, rs;
  VecI cc;
    
  /* zero number of atoms in cells */
  for (i = 0; i < avCells.x * avCells.y; i++) {
    avn[i] = 0;
    avr[i].x = 0.0;
    avr[i].y = 0.0;
    avv[i].x = 0.0;
    avv[i].y = 0.0;
    avatype[i]=0;
  }
  VDiv(invWid, avCells, simArea);
 
  /* find the cells of the atoms and calculate net value of argument */
  for (i = 0; i < nAtom; i++) {
    VSAdd(rs, r[i], 0.5, simArea);
    VMul(cc, rs, invWid);
    c = VLinear(cc, avCells);
    avr[c].x += r[i].x;
    avr[c].y += r[i].y;
    avv[c].x += rArg[i].x;
    avv[c].y += rArg[i].y;
    avn[c]++;
    if (atype[i]==1 || atype[i]==-1){
      avatype[c]-=1;
    }
    else
      avatype[c]+=1;
  }
  /* divide net values by the number of atoms inside cells */
  for (i = 0; i < avCells.x * avCells.y; i++) {
    if (avn[i] > 1) {
      avr[i].x /= (double)avn[i];
      avr[i].y /= (double)avn[i];
      avv[i].x /= (double)avn[i];
      avv[i].y /= (double)avn[i];
    }
    else if (avn[i]==0)
      avr[i].x = DBL_MAX;
  }
  
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/


/*
 * integration over single timestep
 */
void LeapfrogIntegrate()
{
  int n;
  double vdota;
  static double friction=0.9;
  
  yMin = 0.0;
  yMax = 0.0;
  /*Vertical boundary*/
  if (verticalFlag) {
    for (n = 0; n < nAtom; n++) {
      vdota=0.0;
      if (cool.QuenchFlag) {
	vdota=rv[n].x*ra[n].x+rv[n].y*ra[n].y;
	if (vdota<0.0) rv[n].y*=friction;
      }
      if (fixvel[n]==0)
	rv[n].y += deltaT * ra[n].y;
      r[n].y += deltaT * rv[n].y;
      if (!fixedFlag || atype[n]>0) {
	if (cool.QuenchFlag && vdota<0.0) {
	  if (fixvel[n]==0) rv[n].x*=friction;
	}
	if (fixvel[n]==0) rv[n].x += deltaT * ra[n].x;
	r[n].x += deltaT * rv[n].x;
      } else {
	if (fixvel[n]==0) rv[n].x = 0.0; 
      }
      if (periodicXFlag) {
	if (r[n].x<-simArea2.x) r[n].x+=simArea.x;
	if (r[n].x>=simArea2.x) r[n].x-=simArea.x;
      }
      
      /* find the y-edges of the system */
      if (r[n].y < yMin)
	yMin = r[n].y;
      else if (r[n].y > yMax)
	yMax = r[n].y;
    }
  } else { /* Horisontal boundary*/
    for (n = 0; n < nAtom; n++) {
      vdota=0.0;
      if (cool.QuenchFlag) {
	vdota=rv[n].x*ra[n].x+rv[n].y*ra[n].y;
	if (vdota<0.0) {if (fixvel[n]==0) rv[n].x*=friction;}
      }
      if (fixvel[n]==0) rv[n].x += deltaT * ra[n].x;
      r[n].x += deltaT * rv[n].x;
      if (!fixedFlag || atype[n]>0) {
	if (vdota<0.0) {if (fixvel[n]==0) rv[n].y*=friction;}
	if (fixvel[n]==0) rv[n].y += deltaT * ra[n].y;
	r[n].y += deltaT * rv[n].y;
      } else {
	if (fixvel[n]==0) rv[n].y = 0.0; 
      }
      
      if (periodicXFlag) {
	if (r[n].x<-simArea2.x) r[n].x+=simArea.x;
	if (r[n].x>=simArea2.x) r[n].x-=simArea.x;
      }
      
      /* find the y-edges of the system */
      if (r[n].y < yMin)
	yMin = r[n].y;
      else if (r[n].y > yMax)
	yMax = r[n].y;
    }
  }
}  


/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/

  
/* change temperature */
/* We take the square root of the scaling factor here. */

void SetNewTemp(double f)
{
  double vvSum;
  int n;
  
  if (nAtom==0) return;

  if (f<0.0)
    f=0.0;
  else 
    f=sqrt(f);

  vvSum = 0.;
  DO_ATOM {
    if (fixvel[n]==0) VScale(rv[n], f);
    vvSum += VLenSq(rv[n]);
  }

  temperature = UNITTEMP * vvSum / (2.0 * nAtom);

}


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/

/*this is called when the user presses randomize button*/

void RandomizeVelocities()
{
  int n;
  double std;
  VecD vSum;

  VZero(vSum);
  std=sqrt(temperature/UNITTEMP);
  DO_ATOM {
    if (fixvel[n]==0) {
      rv[n].x=std*gasdev(&seed);
      rv[n].y=std*gasdev(&seed);
    }
    VAdd(vSum, vSum, rv[n]);
  }
}


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/

/*
 * initialize atom velocities
 */
void InitialState()
{
  VecD vSum;
  double ang, vMag;
  int n;
  
  stepCount = 0;

  vMag = sqrt(2. * temperature);
  VZero(vSum);
  DO_ATOM {
    /* velocity has fixed magnitude and random direction */
    ang = 2. * M_PI * lcg(&seed); /* drand48(); */
    
    fixvel[n]=0;
    rv[n].x = cos(ang);
    rv[n].y = sin(ang);
    VScale(rv[n], vMag);
    VAdd(vSum, vSum, rv[n]);
    
  }
  /* ensure zero center of mass velocity */
  DO_ATOM {
    /*if (fixvel[n]==0)*/
      VVSAdd(rv[n], - 1. / nAtom, vSum);
  }
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/


/* Random number generator */

double lcg(long int *seed)
{
  static long int a=69069,c=1; 
  /*static long int m=4294967296; original value, risk of overflow*/
  static long int m=2147483647; 
  /*static double rm=4294967296.0; original*/
  static double rm=2147483647.0; 
  *seed=(*seed*a+c)%m;
  return (float)*seed/rm;
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

double gasdev(long int *seed)
{
  static int iset=0;
  static double gset;
  double fac,rsq,v1,v2;
  
  if  (iset == 0) {
    do {
      v1=2.0*lcg(seed)-1.0;
      v2=2.0*lcg(seed)-1.0;
      rsq=v1*v1+v2*v2;
    } while (rsq >= 1.0 || rsq == 0.0);
    fac=sqrt(-2.0*log(rsq)/rsq);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } else {
    iset=0;
    return gset;
  }
}


/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/


/*
 * compute system average kinetic and potential energy and temperature
 */
void CalcEnergy()
{
  VecD v;
  double vvSum,vsq,accel;
  int n;
  
  kinEnmin=1.0e20;
  kinEnmax=-1.0e20;
  forcemin=1.0e20;
  forcemax=-1.0e20;
  vvSum = 0.0;
  DO_ATOM {
    if (fabs(r[n].x) <= 0.5 * simArea.x && fabs(r[n].y) <= 0.5 * simArea.y) {
      /* allow for half timestep */
      VSAdd(v, rv[n], -0.5 * deltaT, ra[n]);
      vsq=VLenSq(v);
      vvSum += vsq;
      accel=sqrt(VLenSq(ra[n]));
      if (vsq<kinEnmin) kinEnmin=vsq;
      if (vsq>kinEnmax) kinEnmax=vsq;
      if (accel<forcemin) forcemin=accel;
      if (accel>forcemax) forcemax=accel;
    }
  }
  
  if (nAtom > 0) {
    kinEnergy = 0.5 * vvSum / nAtom;  
    temperature = UNITTEMP * kinEnergy;
    potEnergy /= nAtom;
    kinEnmin*=0.5;
    kinEnmax*=0.5;
  }
  else {
    kinEnergy = 0.0;
    potEnergy = 0.0;
    kinEnmin=-1.0;
    kinEnmax=1.0;    
  }
}



#ifdef ONEBODY
/* One-body energy and force */
double OneBody(double x, double y, double *e, double *fx, double *fy) 
{
  double sx,cx,sy,cy;
  static double E=5.0,a=0.2,b=0.2;
  sx=sin(a*x);
  cx=cos(a*x);
  sy=sin(b*y);
  cy=cos(b*y);
  *e=E*sx*cy;
  *fx=-E*a*cx*cy;
  *fy=E*b*sx*sy;
}
#endif
